// FILE: ./assets/js/utilities/MYOUtilities.js
var MYOUtilities = function(ScreenService, templateManager, General, $rootScope, ZHPromise, ZhFilterPreferences, ZhPreferences, Query, metaDataFactory) {

    //Filter
    ZhFilterPreferences.oldApplyFilter = ZhFilterPreferences.applyFilter;
    ZhFilterPreferences.applyFilter = function(widget, filter, reloadWidget) {
        return ZhFilterPreferences.oldApplyFilter(widget, filter, reloadWidget).then(function(widgetPromiseReturn) {
            if (reloadWidget !== false) {
                updateStripe(widget);
            }
        }.bind(this));
    };

    var updateStripe = function(gridWidget) {
        return handleStripe(gridWidget).then(function() {
            var filterInfo = gridWidget.__filterInfoStripe;
            deleteUsedFilterParameterOnStripeFields(filterInfo);
            setFieldsByFilterOnStripe(gridWidget._filterInfo, filterInfo);
            hideFieldsNotUsedOnStripe(filterInfo);
            activateCollapseButtonByVisibleFieldsOnStripe(filterInfo);
        });
    };
    this.updateStripe = updateStripe;

    /**
     * Deleta o parâmetro que indica quando uma field do stripe
     * já foi usada para exibir indormações do filtro definido
     * (em todos os fields do stripe).
     * 
     * @param {object} stripe widgtet stripe.
     */
    function deleteUsedFilterParameterOnStripeFields(stripe) {

        stripe.fields.forEach(function(field) {

            if (field.hasOwnProperty("usedFieldOnStripe")) {
                delete field.usedFieldOnStripe;
            }

        });

    }

    /**
     * Define a rotina para exibir os valores filtrados
     * no stripe, onde "OBJECTS_BY_LABEL_ON_STRIPE" e a variável
     * que define o máximo de informações do filtro atual a serem
     * setadas por field.
     * 
     * @param {object} filterInfo informações do filtro
     * definido na widget.
     * 
     * @param {object} stripe widgtet stripe.
     */
    function setFieldsByFilterOnStripe(filterInfo, stripe) {

        if (!Util.isEmptyOrBlank(filterInfo)) {

            var validfilterInfo = getValidFilterInfo(filterInfo);

            var OBJECTS_BY_LABEL_ON_STRIPE = 3;
            var filterInfoLength = validfilterInfo.length;

            if (filterInfoLength > OBJECTS_BY_LABEL_ON_STRIPE) {

                buildStripeField(validfilterInfo.slice(0, 3), stripe);
                setFieldsByFilterOnStripe(validfilterInfo.slice(3), stripe);

            } else if (filterInfoLength > 0) {
                buildStripeField(validfilterInfo, stripe);
            }

        }

    }

    /**
     * Retonar um array com os valores válidos
     * a serem definidos no stripe, ou seja, retorna
     * apenas as informações que possuem label.
     * 
     * @param {object} filterInfo informações do filtro
     * definido na widget.
     */
    function getValidFilterInfo(filterInfo) {

        let labelsToExclude = ["TODAS", "TODOS"];
        let validFilterInfo = [];
        if (isSavedFilter(filterInfo)) {
            validFilterInfo.push({
                label: "Filtro Aplicado",
                value: filterInfo[0].filterName
            });
        } else {
            validFilterInfo = filterInfo.filter(function(filterInfo) {
                let valueIsFilled = (!Util.isEmptyOrBlank(filterInfo.value));
                let labelIsFilled = (!Util.isEmptyOrBlank(filterInfo.label));

                let upperCaseLabel = filterInfo.label.toUpperCase();
                let labelIsNotInExcludeList = (labelsToExclude.indexOf(upperCaseLabel) == -1);

                return (valueIsFilled && labelIsFilled && labelIsNotInExcludeList);
            });
        }
        return validFilterInfo;
    }

    function isSavedFilter(filterInfo) {
        return filterInfo.length && filterInfo[0] && filterInfo[0].filterName;
    }

    /**
     * Cria um field, ou utiliza um já existente,
     * para ser definido no stripe.
     * 
     * @param {object} filterInfo informações do filtro
     * definido na widget.
     * 
     * @param {object} stripe widgtet stripe.
     */
    function buildStripeField(filterInfo, stripe) {

        var stripeFields = stripe.fields;
        var availableFieldIndex = getAvailableFieldIndex(stripeFields);

        if (availableFieldIndex !== undefined) {

            var stripeField = stripeFields[availableFieldIndex];
            var fieldLabel = buildStripeFielLabelByFilterInfo(filterInfo, stripe);
            stripeField.label = fieldLabel;

            if (stripeField.isVisible === false) {
                stripeField.isVisible = true;
            }

        } else {
            stripeFields.push(buildFieldForStripe(filterInfo, stripe));
        }

    }

    /**
     * Retonar o index de um field que ainda não foi utilizado,
     * no stripe, para exibir valores do filtro.
     * 
     * @param {object} stripeFields fields existentes no stripe.
     */
    function getAvailableFieldIndex(stripeFields) {

        if (stripeFields !== undefined && stripeFields.length > 0) {
            for (var i = 0; i < stripeFields.length; i++) {
                if (!Boolean(stripeFields[i].usedFieldOnStripe)) {
                    stripeFields[i].usedFieldOnStripe = true;
                    return i;
                }
            }
        }
        return undefined;
    }

    /**
     * Cria um field para exibir informações do filtro
     * no stripe.
     * 
     * @param {object} filterInfo informações do filtro
     * definido na widget.
     * 
     * @param {object} stripe widgtet stripe.
     */
    function buildFieldForStripe(filterInfo, stripe) {

        var fieldLabel = buildStripeFielLabelByFilterInfo(filterInfo);

        var field = {
            name: "STRIPE_FIELD",
            label: fieldLabel,
            widget: stripe,
            usedFieldOnStripe: true
        };

        return metaDataFactory.fieldFactory(field);

    }

    /**
     * Cria o texto que será definido na label de um field
     * no stripe.
     * 
     * @param {object} filterInfo informações do filtro
     * que serão utilizadas para montar o texto.
     */
    function buildStripeFielLabelByFilterInfo(filterInfo) {

        var fieldLabel;

        if (filterInfo.length === 1) {
            fieldLabel = ScreenService.i18n(filterInfo[0].label) + ": " + filterInfo[0].value;
        } else {

            fieldLabel = filterInfo.reduce(function(acc, cur) {
                return acc + ScreenService.i18n(cur.label) + ": " + cur.value + "   •   ";
            }, "");
            fieldLabel = fieldLabel.replace(/   •   ([^   •   ]*)$/, '$1');

        }

        return fieldLabel;

    }

    /**
     * Oculta fields que não são usandos no stripe.
     * 
     * @param {object} stripe widget stripe.
     */
    function hideFieldsNotUsedOnStripe(stripe) {

        stripe.fields.forEach(function(field) {

            if (!field.hasOwnProperty("usedFieldOnStripe")) {
                field.isVisible = false;
            }

        });

    }

    function activateCollapseButtonByVisibleFieldsOnStripe(stripe) {

        var countStripeVisibleFields = 0;
        var NUMBER_OF_FIELDS_TO_SHOW_COLLAPSE_BUTTON = 3;
        var stripeFields = stripe.fields;

        stripeFields.forEach(function(field) {

            if (field.isVisible) {
                countStripeVisibleFields++;
            }

        });

        if (countStripeVisibleFields >= NUMBER_OF_FIELDS_TO_SHOW_COLLAPSE_BUTTON) {
            stripe.collapsible = true;
        } else {
            stripe.collapsible = false;
        }

    }

    var handleStripe = function(gridWidget) {
        var deferred = ZHPromise.defer();
        if (!gridWidget.__filterInfoStripe) {
            var stripeName = gridWidget.stripeToShowGridFilterInfo;
            if (stripeName) {
                var stripeWidget = templateManager.container.getWidget(stripeName);
                if (stripeWidget) {
                    if (~stripeWidget.template.indexOf("stripe.html")) {
                        var stripe = gridWidget.__filterInfoStripe = stripeWidget;
                        stripe.__stripeOriginalFields = stripe.fields;
                        factoryStripeFields([], gridWidget).then(function(fields) {
                            stripe.fields = fields;
                            deferred.resolve();
                        });
                    } else {
                        console.error("ZhGridFilter: Widget " + stripeName + " is not a Stripe");
                        deferred.reject();
                    }
                } else {
                    console.error("ZhGridFilter: There is no widget named " + stripeName);
                    deferred.reject();
                }
            }
        } else {
            deferred.resolve();
        }
        return deferred.promise;
    };

    var factoryStripeFields = function(fields, gridWidget) {
        var deferred = ZHPromise.defer();
        var stripe = gridWidget.__filterInfoStripe;
        fields = stripe.__stripeOriginalFields ? fields.concat(stripe.__stripeOriginalFields) : fields;
        var fieldsMap = fields.map(function(field) {
            var factoredField = metaDataFactory.fieldFactory(field);
            factoredField.widget = stripe;
            return factoredField;
        });
        deferred.resolve(fields);
        return deferred.promise;
    };

    this.startFilter = function(widget, currentRow) {
        return ZhPreferences.getDefaultWidgetPreferences(widget.id).then(function(result) {
            if (result && result.DEFAULT_FILTER) {
                ZhPreferences.getPreferences('filter', Query.build().where('WIDGET_ID').equals(widget.id)).then(function(filters) {
                    var defaultFilter = filters.filter(function(filter) {
                        return filter.ID == result.DEFAULT_FILTER;
                    })[0];
                    var filter = {
                        ID: defaultFilter.LABEL_TO_PREFERENCE || 'EDITED',
                        FILTER_ROW: JSON.parse(defaultFilter.FILTER_ROW)
                    };

                    currentRow = filter.FILTER_ROW;

                    ZhFilterPreferences.applyFilter(widget, filter).then(function() {
                        widget.appliedFilter = widget.appliedFilter || filter;
                        widget.appliedFilter.ID = result.DEFAULT_FILTER;
                        if (defaultFilter.LABEL_TO_PREFERENCE) {
                            widget._filterInfo[0].value = defaultFilter.LABEL_TO_PREFERENCE;
                        }
                    });
                });
            } else {
                if (Util.isUndefinedOrNull(currentRow) || Util.isEmptyObject(currentRow)) {
                    currentRow = buildFilterCurrentRow(widget);
                }
            }
            ZhFilterPreferences.openFilterWidget(widget, currentRow);
        });
    };

    var buildFilterCurrentRow = function(widget) {
        return getFilterRowWithDefaultValues(widget.fields);
    };

    var getFilterRowWithDefaultValues = function(filterFields) {
        let filterRow = {};
        filterFields.forEach(field => {
            let defaultValue = Util.getNestedProperty(field, ["filterProperties", "defaultValue"]) || field.defaultValue;
            if (!Util.isUndefinedOrNull(defaultValue)) {
                filterRow[field.name] = Util.evaluate(defaultValue);
            }
        });
        return filterRow;
    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register('MYOUtilities', MYOUtilities);
});

// FILE: ./assets/js/utilities/QrCodeGenerator.js
/* jshint ignore:start */
/**
 * @fileoverview
 * - Using the 'QRCode for Javascript library'
 * - Fixed dataset of 'QRCode for Javascript library' for support full-spec.
 * - this library has no dependencies.
 * 
 * @author davidshimjs
 * @see <a href="http://www.d-project.com/" target="_blank">http://www.d-project.com/</a>
 * @see <a href="http://jeromeetienne.github.com/jquery-qrcode/" target="_blank">http://jeromeetienne.github.com/jquery-qrcode/</a>
 */
var QRCode;

(function() {
    //---------------------------------------------------------------------
    // QRCode for JavaScript
    //
    // Copyright (c) 2009 Kazuhiko Arase
    //
    // URL: http://www.d-project.com/
    //
    // Licensed under the MIT license:
    //   http://www.opensource.org/licenses/mit-license.php
    //
    // The word "QR Code" is registered trademark of 
    // DENSO WAVE INCORPORATED
    //   http://www.denso-wave.com/qrcode/faqpatent-e.html
    //
    //---------------------------------------------------------------------
    function QR8bitByte(data) {
        this.mode = QRMode.MODE_8BIT_BYTE;
        this.data = data;
        this.parsedData = [];

        // Added to support UTF-8 Characters
        for (var i = 0, l = this.data.length; i < l; i++) {
            var byteArray = [];
            var code = this.data.charCodeAt(i);

            if (code > 0x10000) {
                byteArray[0] = 0xF0 | ((code & 0x1C0000) >>> 18);
                byteArray[1] = 0x80 | ((code & 0x3F000) >>> 12);
                byteArray[2] = 0x80 | ((code & 0xFC0) >>> 6);
                byteArray[3] = 0x80 | (code & 0x3F);
            } else if (code > 0x800) {
                byteArray[0] = 0xE0 | ((code & 0xF000) >>> 12);
                byteArray[1] = 0x80 | ((code & 0xFC0) >>> 6);
                byteArray[2] = 0x80 | (code & 0x3F);
            } else if (code > 0x80) {
                byteArray[0] = 0xC0 | ((code & 0x7C0) >>> 6);
                byteArray[1] = 0x80 | (code & 0x3F);
            } else {
                byteArray[0] = code;
            }

            this.parsedData.push(byteArray);
        }

        this.parsedData = Array.prototype.concat.apply([], this.parsedData);

        if (this.parsedData.length != this.data.length) {
            this.parsedData.unshift(191);
            this.parsedData.unshift(187);
            this.parsedData.unshift(239);
        }
    }

    QR8bitByte.prototype = {
        getLength: function(buffer) {
            return this.parsedData.length;
        },
        write: function(buffer) {
            for (var i = 0, l = this.parsedData.length; i < l; i++) {
                buffer.put(this.parsedData[i], 8);
            }
        }
    };

    function QRCodeModel(typeNumber, errorCorrectLevel) {
        this.typeNumber = typeNumber;
        this.errorCorrectLevel = errorCorrectLevel;
        this.modules = null;
        this.moduleCount = 0;
        this.dataCache = null;
        this.dataList = [];
    }

    QRCodeModel.prototype = {
        addData: function(data) {
            var newData = new QR8bitByte(data);
            this.dataList.push(newData);
            this.dataCache = null;
        },
        isDark: function(row, col) {
            if (row < 0 || this.moduleCount <= row || col < 0 || this.moduleCount <= col) {
                throw new Error(row + "," + col);
            }
            return this.modules[row][col];
        },
        getModuleCount: function() {
            return this.moduleCount;
        },
        make: function() {
            this.makeImpl(false, this.getBestMaskPattern());
        },
        makeImpl: function(test, maskPattern) {
            this.moduleCount = this.typeNumber * 4 + 17;
            this.modules = new Array(this.moduleCount);
            for (var row = 0; row < this.moduleCount; row++) {
                this.modules[row] = new Array(this.moduleCount);
                for (var col = 0; col < this.moduleCount; col++) {
                    this.modules[row][col] = null;
                }
            }
            this.setupPositionProbePattern(0, 0);
            this.setupPositionProbePattern(this.moduleCount - 7, 0);
            this.setupPositionProbePattern(0, this.moduleCount - 7);
            this.setupPositionAdjustPattern();
            this.setupTimingPattern();
            this.setupTypeInfo(test, maskPattern);
            if (this.typeNumber >= 7) {
                this.setupTypeNumber(test);
            }
            if (this.dataCache == null) {
                this.dataCache = QRCodeModel.createData(this.typeNumber, this.errorCorrectLevel, this.dataList);
            }
            this.mapData(this.dataCache, maskPattern);
        },
        setupPositionProbePattern: function(row, col) {
            for (var r = -1; r <= 7; r++) {
                if (row + r <= -1 || this.moduleCount <= row + r) continue;
                for (var c = -1; c <= 7; c++) {
                    if (col + c <= -1 || this.moduleCount <= col + c) continue;
                    if ((0 <= r && r <= 6 && (c == 0 || c == 6)) || (0 <= c && c <= 6 && (r == 0 || r == 6)) || (2 <= r && r <= 4 && 2 <= c && c <= 4)) {
                        this.modules[row + r][col + c] = true;
                    } else {
                        this.modules[row + r][col + c] = false;
                    }
                }
            }
        },
        getBestMaskPattern: function() {
            var minLostPoint = 0;
            var pattern = 0;
            for (var i = 0; i < 8; i++) {
                this.makeImpl(true, i);
                var lostPoint = QRUtil.getLostPoint(this);
                if (i == 0 || minLostPoint > lostPoint) {
                    minLostPoint = lostPoint;
                    pattern = i;
                }
            }
            return pattern;
        },
        createMovieClip: function(target_mc, instance_name, depth) {
            var qr_mc = target_mc.createEmptyMovieClip(instance_name, depth);
            var cs = 1;
            this.make();
            for (var row = 0; row < this.modules.length; row++) {
                var y = row * cs;
                for (var col = 0; col < this.modules[row].length; col++) {
                    var x = col * cs;
                    var dark = this.modules[row][col];
                    if (dark) {
                        qr_mc.beginFill(0, 100);
                        qr_mc.moveTo(x, y);
                        qr_mc.lineTo(x + cs, y);
                        qr_mc.lineTo(x + cs, y + cs);
                        qr_mc.lineTo(x, y + cs);
                        qr_mc.endFill();
                    }
                }
            }
            return qr_mc;
        },
        setupTimingPattern: function() {
            for (var r = 8; r < this.moduleCount - 8; r++) {
                if (this.modules[r][6] != null) {
                    continue;
                }
                this.modules[r][6] = (r % 2 == 0);
            }
            for (var c = 8; c < this.moduleCount - 8; c++) {
                if (this.modules[6][c] != null) {
                    continue;
                }
                this.modules[6][c] = (c % 2 == 0);
            }
        },
        setupPositionAdjustPattern: function() {
            var pos = QRUtil.getPatternPosition(this.typeNumber);
            for (var i = 0; i < pos.length; i++) {
                for (var j = 0; j < pos.length; j++) {
                    var row = pos[i];
                    var col = pos[j];
                    if (this.modules[row][col] != null) {
                        continue;
                    }
                    for (var r = -2; r <= 2; r++) {
                        for (var c = -2; c <= 2; c++) {
                            if (r == -2 || r == 2 || c == -2 || c == 2 || (r == 0 && c == 0)) {
                                this.modules[row + r][col + c] = true;
                            } else {
                                this.modules[row + r][col + c] = false;
                            }
                        }
                    }
                }
            }
        },
        setupTypeNumber: function(test) {
            var bits = QRUtil.getBCHTypeNumber(this.typeNumber);
            for (var i = 0; i < 18; i++) {
                var mod = (!test && ((bits >> i) & 1) == 1);
                this.modules[Math.floor(i / 3)][i % 3 + this.moduleCount - 8 - 3] = mod;
            }
            for (var i = 0; i < 18; i++) {
                var mod = (!test && ((bits >> i) & 1) == 1);
                this.modules[i % 3 + this.moduleCount - 8 - 3][Math.floor(i / 3)] = mod;
            }
        },
        setupTypeInfo: function(test, maskPattern) {
            var data = (this.errorCorrectLevel << 3) | maskPattern;
            var bits = QRUtil.getBCHTypeInfo(data);
            for (var i = 0; i < 15; i++) {
                var mod = (!test && ((bits >> i) & 1) == 1);
                if (i < 6) {
                    this.modules[i][8] = mod;
                } else if (i < 8) {
                    this.modules[i + 1][8] = mod;
                } else {
                    this.modules[this.moduleCount - 15 + i][8] = mod;
                }
            }
            for (var i = 0; i < 15; i++) {
                var mod = (!test && ((bits >> i) & 1) == 1);
                if (i < 8) {
                    this.modules[8][this.moduleCount - i - 1] = mod;
                } else if (i < 9) {
                    this.modules[8][15 - i - 1 + 1] = mod;
                } else {
                    this.modules[8][15 - i - 1] = mod;
                }
            }
            this.modules[this.moduleCount - 8][8] = (!test);
        },
        mapData: function(data, maskPattern) {
            var inc = -1;
            var row = this.moduleCount - 1;
            var bitIndex = 7;
            var byteIndex = 0;
            for (var col = this.moduleCount - 1; col > 0; col -= 2) {
                if (col == 6) col--;
                while (true) {
                    for (var c = 0; c < 2; c++) {
                        if (this.modules[row][col - c] == null) {
                            var dark = false;
                            if (byteIndex < data.length) {
                                dark = (((data[byteIndex] >>> bitIndex) & 1) == 1);
                            }
                            var mask = QRUtil.getMask(maskPattern, row, col - c);
                            if (mask) {
                                dark = !dark;
                            }
                            this.modules[row][col - c] = dark;
                            bitIndex--;
                            if (bitIndex == -1) {
                                byteIndex++;
                                bitIndex = 7;
                            }
                        }
                    }
                    row += inc;
                    if (row < 0 || this.moduleCount <= row) {
                        row -= inc;
                        inc = -inc;
                        break;
                    }
                }
            }
        }
    };
    QRCodeModel.PAD0 = 0xEC;
    QRCodeModel.PAD1 = 0x11;
    QRCodeModel.createData = function(typeNumber, errorCorrectLevel, dataList) {
        var rsBlocks = QRRSBlock.getRSBlocks(typeNumber, errorCorrectLevel);
        var buffer = new QRBitBuffer();
        for (var i = 0; i < dataList.length; i++) {
            var data = dataList[i];
            buffer.put(data.mode, 4);
            buffer.put(data.getLength(), QRUtil.getLengthInBits(data.mode, typeNumber));
            data.write(buffer);
        }
        var totalDataCount = 0;
        for (var i = 0; i < rsBlocks.length; i++) {
            totalDataCount += rsBlocks[i].dataCount;
        }
        if (buffer.getLengthInBits() > totalDataCount * 8) {
            throw new Error("code length overflow. (" +
                buffer.getLengthInBits() +
                ">" +
                totalDataCount * 8 +
                ")");
        }
        if (buffer.getLengthInBits() + 4 <= totalDataCount * 8) {
            buffer.put(0, 4);
        }
        while (buffer.getLengthInBits() % 8 != 0) {
            buffer.putBit(false);
        }
        while (true) {
            if (buffer.getLengthInBits() >= totalDataCount * 8) {
                break;
            }
            buffer.put(QRCodeModel.PAD0, 8);
            if (buffer.getLengthInBits() >= totalDataCount * 8) {
                break;
            }
            buffer.put(QRCodeModel.PAD1, 8);
        }
        return QRCodeModel.createBytes(buffer, rsBlocks);
    };
    QRCodeModel.createBytes = function(buffer, rsBlocks) {
        var offset = 0;
        var maxDcCount = 0;
        var maxEcCount = 0;
        var dcdata = new Array(rsBlocks.length);
        var ecdata = new Array(rsBlocks.length);
        for (var r = 0; r < rsBlocks.length; r++) {
            var dcCount = rsBlocks[r].dataCount;
            var ecCount = rsBlocks[r].totalCount - dcCount;
            maxDcCount = Math.max(maxDcCount, dcCount);
            maxEcCount = Math.max(maxEcCount, ecCount);
            dcdata[r] = new Array(dcCount);
            for (var i = 0; i < dcdata[r].length; i++) {
                dcdata[r][i] = 0xff & buffer.buffer[i + offset];
            }
            offset += dcCount;
            var rsPoly = QRUtil.getErrorCorrectPolynomial(ecCount);
            var rawPoly = new QRPolynomial(dcdata[r], rsPoly.getLength() - 1);
            var modPoly = rawPoly.mod(rsPoly);
            ecdata[r] = new Array(rsPoly.getLength() - 1);
            for (var i = 0; i < ecdata[r].length; i++) {
                var modIndex = i + modPoly.getLength() - ecdata[r].length;
                ecdata[r][i] = (modIndex >= 0) ? modPoly.get(modIndex) : 0;
            }
        }
        var totalCodeCount = 0;
        for (var i = 0; i < rsBlocks.length; i++) {
            totalCodeCount += rsBlocks[i].totalCount;
        }
        var data = new Array(totalCodeCount);
        var index = 0;
        for (var i = 0; i < maxDcCount; i++) {
            for (var r = 0; r < rsBlocks.length; r++) {
                if (i < dcdata[r].length) {
                    data[index++] = dcdata[r][i];
                }
            }
        }
        for (var i = 0; i < maxEcCount; i++) {
            for (var r = 0; r < rsBlocks.length; r++) {
                if (i < ecdata[r].length) {
                    data[index++] = ecdata[r][i];
                }
            }
        }
        return data;
    };
    var QRMode = {
        MODE_NUMBER: 1 << 0,
        MODE_ALPHA_NUM: 1 << 1,
        MODE_8BIT_BYTE: 1 << 2,
        MODE_KANJI: 1 << 3
    };
    var QRErrorCorrectLevel = {
        L: 1,
        M: 0,
        Q: 3,
        H: 2
    };
    var QRMaskPattern = {
        PATTERN000: 0,
        PATTERN001: 1,
        PATTERN010: 2,
        PATTERN011: 3,
        PATTERN100: 4,
        PATTERN101: 5,
        PATTERN110: 6,
        PATTERN111: 7
    };
    var QRUtil = {
        PATTERN_POSITION_TABLE: [[], [6, 18], [6, 22], [6, 26], [6, 30], [6, 34], [6, 22, 38], [6, 24, 42], [6, 26, 46], [6, 28, 50], [6, 30, 54], [6, 32, 58], [6, 34, 62], [6, 26, 46, 66], [6, 26, 48, 70], [6, 26, 50, 74], [6, 30, 54, 78], [6, 30, 56, 82], [6, 30, 58, 86], [6, 34, 62, 90], [6, 28, 50, 72, 94], [6, 26, 50, 74, 98], [6, 30, 54, 78, 102], [6, 28, 54, 80, 106], [6, 32, 58, 84, 110], [6, 30, 58, 86, 114], [6, 34, 62, 90, 118], [6, 26, 50, 74, 98, 122], [6, 30, 54, 78, 102, 126], [6, 26, 52, 78, 104, 130], [6, 30, 56, 82, 108, 134], [6, 34, 60, 86, 112, 138], [6, 30, 58, 86, 114, 142], [6, 34, 62, 90, 118, 146], [6, 30, 54, 78, 102, 126, 150], [6, 24, 50, 76, 102, 128, 154], [6, 28, 54, 80, 106, 132, 158], [6, 32, 58, 84, 110, 136, 162], [6, 26, 54, 82, 110, 138, 166], [6, 30, 58, 86, 114, 142, 170]],
        G15: (1 << 10) | (1 << 8) | (1 << 5) | (1 << 4) | (1 << 2) | (1 << 1) | (1 << 0),
        G18: (1 << 12) | (1 << 11) | (1 << 10) | (1 << 9) | (1 << 8) | (1 << 5) | (1 << 2) | (1 << 0),
        G15_MASK: (1 << 14) | (1 << 12) | (1 << 10) | (1 << 4) | (1 << 1),
        getBCHTypeInfo: function(data) {
            var d = data << 10;
            while (QRUtil.getBCHDigit(d) - QRUtil.getBCHDigit(QRUtil.G15) >= 0) {
                d ^= (QRUtil.G15 << (QRUtil.getBCHDigit(d) - QRUtil.getBCHDigit(QRUtil.G15)));
            }
            return ((data << 10) | d) ^ QRUtil.G15_MASK;
        },
        getBCHTypeNumber: function(data) {
            var d = data << 12;
            while (QRUtil.getBCHDigit(d) - QRUtil.getBCHDigit(QRUtil.G18) >= 0) {
                d ^= (QRUtil.G18 << (QRUtil.getBCHDigit(d) - QRUtil.getBCHDigit(QRUtil.G18)));
            }
            return (data << 12) | d;
        },
        getBCHDigit: function(data) {
            var digit = 0;
            while (data != 0) {
                digit++;
                data >>>= 1;
            }
            return digit;
        },
        getPatternPosition: function(typeNumber) {
            return QRUtil.PATTERN_POSITION_TABLE[typeNumber - 1];
        },
        getMask: function(maskPattern, i, j) {
            switch (maskPattern) {
                case QRMaskPattern.PATTERN000:
                    return (i + j) % 2 == 0;
                case QRMaskPattern.PATTERN001:
                    return i % 2 == 0;
                case QRMaskPattern.PATTERN010:
                    return j % 3 == 0;
                case QRMaskPattern.PATTERN011:
                    return (i + j) % 3 == 0;
                case QRMaskPattern.PATTERN100:
                    return (Math.floor(i / 2) + Math.floor(j / 3)) % 2 == 0;
                case QRMaskPattern.PATTERN101:
                    return (i * j) % 2 + (i * j) % 3 == 0;
                case QRMaskPattern.PATTERN110:
                    return ((i * j) % 2 + (i * j) % 3) % 2 == 0;
                case QRMaskPattern.PATTERN111:
                    return ((i * j) % 3 + (i + j) % 2) % 2 == 0;
                default:
                    throw new Error("bad maskPattern:" + maskPattern);
            }
        },
        getErrorCorrectPolynomial: function(errorCorrectLength) {
            var a = new QRPolynomial([1], 0);
            for (var i = 0; i < errorCorrectLength; i++) {
                a = a.multiply(new QRPolynomial([1, QRMath.gexp(i)], 0));
            }
            return a;
        },
        getLengthInBits: function(mode, type) {
            if (1 <= type && type < 10) {
                switch (mode) {
                    case QRMode.MODE_NUMBER:
                        return 10;
                    case QRMode.MODE_ALPHA_NUM:
                        return 9;
                    case QRMode.MODE_8BIT_BYTE:
                        return 8;
                    case QRMode.MODE_KANJI:
                        return 8;
                    default:
                        throw new Error("mode:" + mode);
                }
            } else if (type < 27) {
                switch (mode) {
                    case QRMode.MODE_NUMBER:
                        return 12;
                    case QRMode.MODE_ALPHA_NUM:
                        return 11;
                    case QRMode.MODE_8BIT_BYTE:
                        return 16;
                    case QRMode.MODE_KANJI:
                        return 10;
                    default:
                        throw new Error("mode:" + mode);
                }
            } else if (type < 41) {
                switch (mode) {
                    case QRMode.MODE_NUMBER:
                        return 14;
                    case QRMode.MODE_ALPHA_NUM:
                        return 13;
                    case QRMode.MODE_8BIT_BYTE:
                        return 16;
                    case QRMode.MODE_KANJI:
                        return 12;
                    default:
                        throw new Error("mode:" + mode);
                }
            } else {
                throw new Error("type:" + type);
            }
        },
        getLostPoint: function(qrCode) {
            var moduleCount = qrCode.getModuleCount();
            var lostPoint = 0;
            for (var row = 0; row < moduleCount; row++) {
                for (var col = 0; col < moduleCount; col++) {
                    var sameCount = 0;
                    var dark = qrCode.isDark(row, col);
                    for (var r = -1; r <= 1; r++) {
                        if (row + r < 0 || moduleCount <= row + r) {
                            continue;
                        }
                        for (var c = -1; c <= 1; c++) {
                            if (col + c < 0 || moduleCount <= col + c) {
                                continue;
                            }
                            if (r == 0 && c == 0) {
                                continue;
                            }
                            if (dark == qrCode.isDark(row + r, col + c)) {
                                sameCount++;
                            }
                        }
                    }
                    if (sameCount > 5) {
                        lostPoint += (3 + sameCount - 5);
                    }
                }
            }
            for (var row = 0; row < moduleCount - 1; row++) {
                for (var col = 0; col < moduleCount - 1; col++) {
                    var count = 0;
                    if (qrCode.isDark(row, col)) count++;
                    if (qrCode.isDark(row + 1, col)) count++;
                    if (qrCode.isDark(row, col + 1)) count++;
                    if (qrCode.isDark(row + 1, col + 1)) count++;
                    if (count == 0 || count == 4) {
                        lostPoint += 3;
                    }
                }
            }
            for (var row = 0; row < moduleCount; row++) {
                for (var col = 0; col < moduleCount - 6; col++) {
                    if (qrCode.isDark(row, col) && !qrCode.isDark(row, col + 1) && qrCode.isDark(row, col + 2) && qrCode.isDark(row, col + 3) && qrCode.isDark(row, col + 4) && !qrCode.isDark(row, col + 5) && qrCode.isDark(row, col + 6)) {
                        lostPoint += 40;
                    }
                }
            }
            for (var col = 0; col < moduleCount; col++) {
                for (var row = 0; row < moduleCount - 6; row++) {
                    if (qrCode.isDark(row, col) && !qrCode.isDark(row + 1, col) && qrCode.isDark(row + 2, col) && qrCode.isDark(row + 3, col) && qrCode.isDark(row + 4, col) && !qrCode.isDark(row + 5, col) && qrCode.isDark(row + 6, col)) {
                        lostPoint += 40;
                    }
                }
            }
            var darkCount = 0;
            for (var col = 0; col < moduleCount; col++) {
                for (var row = 0; row < moduleCount; row++) {
                    if (qrCode.isDark(row, col)) {
                        darkCount++;
                    }
                }
            }
            var ratio = Math.abs(100 * darkCount / moduleCount / moduleCount - 50) / 5;
            lostPoint += ratio * 10;
            return lostPoint;
        }
    };
    var QRMath = {
        glog: function(n) {
            if (n < 1) {
                throw new Error("glog(" + n + ")");
            }
            return QRMath.LOG_TABLE[n];
        },
        gexp: function(n) {
            while (n < 0) {
                n += 255;
            }
            while (n >= 256) {
                n -= 255;
            }
            return QRMath.EXP_TABLE[n];
        },
        EXP_TABLE: new Array(256),
        LOG_TABLE: new Array(256)
    };
    for (var i = 0; i < 8; i++) {
        QRMath.EXP_TABLE[i] = 1 << i;
    }
    for (var i = 8; i < 256; i++) {
        QRMath.EXP_TABLE[i] = QRMath.EXP_TABLE[i - 4] ^ QRMath.EXP_TABLE[i - 5] ^ QRMath.EXP_TABLE[i - 6] ^ QRMath.EXP_TABLE[i - 8];
    }
    for (var i = 0; i < 255; i++) {
        QRMath.LOG_TABLE[QRMath.EXP_TABLE[i]] = i;
    }

    function QRPolynomial(num, shift) {
        if (num.length == undefined) {
            throw new Error(num.length + "/" + shift);
        }
        var offset = 0;
        while (offset < num.length && num[offset] == 0) {
            offset++;
        }
        this.num = new Array(num.length - offset + shift);
        for (var i = 0; i < num.length - offset; i++) {
            this.num[i] = num[i + offset];
        }
    }
    QRPolynomial.prototype = {
        get: function(index) {
            return this.num[index];
        },
        getLength: function() {
            return this.num.length;
        },
        multiply: function(e) {
            var num = new Array(this.getLength() + e.getLength() - 1);
            for (var i = 0; i < this.getLength(); i++) {
                for (var j = 0; j < e.getLength(); j++) {
                    num[i + j] ^= QRMath.gexp(QRMath.glog(this.get(i)) + QRMath.glog(e.get(j)));
                }
            }
            return new QRPolynomial(num, 0);
        },
        mod: function(e) {
            if (this.getLength() - e.getLength() < 0) {
                return this;
            }
            var ratio = QRMath.glog(this.get(0)) - QRMath.glog(e.get(0));
            var num = new Array(this.getLength());
            for (var i = 0; i < this.getLength(); i++) {
                num[i] = this.get(i);
            }
            for (var i = 0; i < e.getLength(); i++) {
                num[i] ^= QRMath.gexp(QRMath.glog(e.get(i)) + ratio);
            }
            return new QRPolynomial(num, 0).mod(e);
        }
    };

    function QRRSBlock(totalCount, dataCount) {
        this.totalCount = totalCount;
        this.dataCount = dataCount;
    }
    QRRSBlock.RS_BLOCK_TABLE = [[1, 26, 19], [1, 26, 16], [1, 26, 13], [1, 26, 9], [1, 44, 34], [1, 44, 28], [1, 44, 22], [1, 44, 16], [1, 70, 55], [1, 70, 44], [2, 35, 17], [2, 35, 13], [1, 100, 80], [2, 50, 32], [2, 50, 24], [4, 25, 9], [1, 134, 108], [2, 67, 43], [2, 33, 15, 2, 34, 16], [2, 33, 11, 2, 34, 12], [2, 86, 68], [4, 43, 27], [4, 43, 19], [4, 43, 15], [2, 98, 78], [4, 49, 31], [2, 32, 14, 4, 33, 15], [4, 39, 13, 1, 40, 14], [2, 121, 97], [2, 60, 38, 2, 61, 39], [4, 40, 18, 2, 41, 19], [4, 40, 14, 2, 41, 15], [2, 146, 116], [3, 58, 36, 2, 59, 37], [4, 36, 16, 4, 37, 17], [4, 36, 12, 4, 37, 13], [2, 86, 68, 2, 87, 69], [4, 69, 43, 1, 70, 44], [6, 43, 19, 2, 44, 20], [6, 43, 15, 2, 44, 16], [4, 101, 81], [1, 80, 50, 4, 81, 51], [4, 50, 22, 4, 51, 23], [3, 36, 12, 8, 37, 13], [2, 116, 92, 2, 117, 93], [6, 58, 36, 2, 59, 37], [4, 46, 20, 6, 47, 21], [7, 42, 14, 4, 43, 15], [4, 133, 107], [8, 59, 37, 1, 60, 38], [8, 44, 20, 4, 45, 21], [12, 33, 11, 4, 34, 12], [3, 145, 115, 1, 146, 116], [4, 64, 40, 5, 65, 41], [11, 36, 16, 5, 37, 17], [11, 36, 12, 5, 37, 13], [5, 109, 87, 1, 110, 88], [5, 65, 41, 5, 66, 42], [5, 54, 24, 7, 55, 25], [11, 36, 12], [5, 122, 98, 1, 123, 99], [7, 73, 45, 3, 74, 46], [15, 43, 19, 2, 44, 20], [3, 45, 15, 13, 46, 16], [1, 135, 107, 5, 136, 108], [10, 74, 46, 1, 75, 47], [1, 50, 22, 15, 51, 23], [2, 42, 14, 17, 43, 15], [5, 150, 120, 1, 151, 121], [9, 69, 43, 4, 70, 44], [17, 50, 22, 1, 51, 23], [2, 42, 14, 19, 43, 15], [3, 141, 113, 4, 142, 114], [3, 70, 44, 11, 71, 45], [17, 47, 21, 4, 48, 22], [9, 39, 13, 16, 40, 14], [3, 135, 107, 5, 136, 108], [3, 67, 41, 13, 68, 42], [15, 54, 24, 5, 55, 25], [15, 43, 15, 10, 44, 16], [4, 144, 116, 4, 145, 117], [17, 68, 42], [17, 50, 22, 6, 51, 23], [19, 46, 16, 6, 47, 17], [2, 139, 111, 7, 140, 112], [17, 74, 46], [7, 54, 24, 16, 55, 25], [34, 37, 13], [4, 151, 121, 5, 152, 122], [4, 75, 47, 14, 76, 48], [11, 54, 24, 14, 55, 25], [16, 45, 15, 14, 46, 16], [6, 147, 117, 4, 148, 118], [6, 73, 45, 14, 74, 46], [11, 54, 24, 16, 55, 25], [30, 46, 16, 2, 47, 17], [8, 132, 106, 4, 133, 107], [8, 75, 47, 13, 76, 48], [7, 54, 24, 22, 55, 25], [22, 45, 15, 13, 46, 16], [10, 142, 114, 2, 143, 115], [19, 74, 46, 4, 75, 47], [28, 50, 22, 6, 51, 23], [33, 46, 16, 4, 47, 17], [8, 152, 122, 4, 153, 123], [22, 73, 45, 3, 74, 46], [8, 53, 23, 26, 54, 24], [12, 45, 15, 28, 46, 16], [3, 147, 117, 10, 148, 118], [3, 73, 45, 23, 74, 46], [4, 54, 24, 31, 55, 25], [11, 45, 15, 31, 46, 16], [7, 146, 116, 7, 147, 117], [21, 73, 45, 7, 74, 46], [1, 53, 23, 37, 54, 24], [19, 45, 15, 26, 46, 16], [5, 145, 115, 10, 146, 116], [19, 75, 47, 10, 76, 48], [15, 54, 24, 25, 55, 25], [23, 45, 15, 25, 46, 16], [13, 145, 115, 3, 146, 116], [2, 74, 46, 29, 75, 47], [42, 54, 24, 1, 55, 25], [23, 45, 15, 28, 46, 16], [17, 145, 115], [10, 74, 46, 23, 75, 47], [10, 54, 24, 35, 55, 25], [19, 45, 15, 35, 46, 16], [17, 145, 115, 1, 146, 116], [14, 74, 46, 21, 75, 47], [29, 54, 24, 19, 55, 25], [11, 45, 15, 46, 46, 16], [13, 145, 115, 6, 146, 116], [14, 74, 46, 23, 75, 47], [44, 54, 24, 7, 55, 25], [59, 46, 16, 1, 47, 17], [12, 151, 121, 7, 152, 122], [12, 75, 47, 26, 76, 48], [39, 54, 24, 14, 55, 25], [22, 45, 15, 41, 46, 16], [6, 151, 121, 14, 152, 122], [6, 75, 47, 34, 76, 48], [46, 54, 24, 10, 55, 25], [2, 45, 15, 64, 46, 16], [17, 152, 122, 4, 153, 123], [29, 74, 46, 14, 75, 47], [49, 54, 24, 10, 55, 25], [24, 45, 15, 46, 46, 16], [4, 152, 122, 18, 153, 123], [13, 74, 46, 32, 75, 47], [48, 54, 24, 14, 55, 25], [42, 45, 15, 32, 46, 16], [20, 147, 117, 4, 148, 118], [40, 75, 47, 7, 76, 48], [43, 54, 24, 22, 55, 25], [10, 45, 15, 67, 46, 16], [19, 148, 118, 6, 149, 119], [18, 75, 47, 31, 76, 48], [34, 54, 24, 34, 55, 25], [20, 45, 15, 61, 46, 16]];
    QRRSBlock.getRSBlocks = function(typeNumber, errorCorrectLevel) {
        var rsBlock = QRRSBlock.getRsBlockTable(typeNumber, errorCorrectLevel);
        if (rsBlock == undefined) {
            throw new Error("bad rs block @ typeNumber:" + typeNumber + "/errorCorrectLevel:" + errorCorrectLevel);
        }
        var length = rsBlock.length / 3;
        var list = [];
        for (var i = 0; i < length; i++) {
            var count = rsBlock[i * 3 + 0];
            var totalCount = rsBlock[i * 3 + 1];
            var dataCount = rsBlock[i * 3 + 2];
            for (var j = 0; j < count; j++) {
                list.push(new QRRSBlock(totalCount, dataCount));
            }
        }
        return list;
    };
    QRRSBlock.getRsBlockTable = function(typeNumber, errorCorrectLevel) {
        switch (errorCorrectLevel) {
            case QRErrorCorrectLevel.L:
                return QRRSBlock.RS_BLOCK_TABLE[(typeNumber - 1) * 4 + 0];
            case QRErrorCorrectLevel.M:
                return QRRSBlock.RS_BLOCK_TABLE[(typeNumber - 1) * 4 + 1];
            case QRErrorCorrectLevel.Q:
                return QRRSBlock.RS_BLOCK_TABLE[(typeNumber - 1) * 4 + 2];
            case QRErrorCorrectLevel.H:
                return QRRSBlock.RS_BLOCK_TABLE[(typeNumber - 1) * 4 + 3];
            default:
                return undefined;
        }
    };

    function QRBitBuffer() {
        this.buffer = [];
        this.length = 0;
    }
    QRBitBuffer.prototype = {
        get: function(index) {
            var bufIndex = Math.floor(index / 8);
            return ((this.buffer[bufIndex] >>> (7 - index % 8)) & 1) == 1;
        },
        put: function(num, length) {
            for (var i = 0; i < length; i++) {
                this.putBit(((num >>> (length - i - 1)) & 1) == 1);
            }
        },
        getLengthInBits: function() {
            return this.length;
        },
        putBit: function(bit) {
            var bufIndex = Math.floor(this.length / 8);
            if (this.buffer.length <= bufIndex) {
                this.buffer.push(0);
            }
            if (bit) {
                this.buffer[bufIndex] |= (0x80 >>> (this.length % 8));
            }
            this.length++;
        }
    };
    var QRCodeLimitLength = [[17, 14, 11, 7], [32, 26, 20, 14], [53, 42, 32, 24], [78, 62, 46, 34], [106, 84, 60, 44], [134, 106, 74, 58], [154, 122, 86, 64], [192, 152, 108, 84], [230, 180, 130, 98], [271, 213, 151, 119], [321, 251, 177, 137], [367, 287, 203, 155], [425, 331, 241, 177], [458, 362, 258, 194], [520, 412, 292, 220], [586, 450, 322, 250], [644, 504, 364, 280], [718, 560, 394, 310], [792, 624, 442, 338], [858, 666, 482, 382], [929, 711, 509, 403], [1003, 779, 565, 439], [1091, 857, 611, 461], [1171, 911, 661, 511], [1273, 997, 715, 535], [1367, 1059, 751, 593], [1465, 1125, 805, 625], [1528, 1190, 868, 658], [1628, 1264, 908, 698], [1732, 1370, 982, 742], [1840, 1452, 1030, 790], [1952, 1538, 1112, 842], [2068, 1628, 1168, 898], [2188, 1722, 1228, 958], [2303, 1809, 1283, 983], [2431, 1911, 1351, 1051], [2563, 1989, 1423, 1093], [2699, 2099, 1499, 1139], [2809, 2213, 1579, 1219], [2953, 2331, 1663, 1273]];

    function _isSupportCanvas() {
        return typeof CanvasRenderingContext2D != "undefined";
    }

    // android 2.x doesn't support Data-URI spec
    function _getAndroid() {
        var android = false;
        var sAgent = navigator.userAgent;

        if (/android/i.test(sAgent)) { // android
            android = true;
            var aMat = sAgent.toString().match(/android ([0-9]\.[0-9])/i);

            if (aMat && aMat[1]) {
                android = parseFloat(aMat[1]);
            }
        }

        return android;
    }

    var svgDrawer = (function() {

        var Drawing = function(el, htOption) {
            this._el = el;
            this._htOption = htOption;
        };

        Drawing.prototype.draw = function(oQRCode) {
            var _htOption = this._htOption;
            var _el = this._el;
            var nCount = oQRCode.getModuleCount();
            var nWidth = Math.floor(_htOption.width / nCount);
            var nHeight = Math.floor(_htOption.height / nCount);

            this.clear();

            function makeSVG(tag, attrs) {
                var el = document.createElementNS('http://www.w3.org/2000/svg', tag);
                for (var k in attrs)
                    if (attrs.hasOwnProperty(k)) el.setAttribute(k, attrs[k]);
                return el;
            }

            var svg = makeSVG("svg", {
                'viewBox': '0 0 ' + String(nCount) + " " + String(nCount),
                'width': '100%',
                'height': '100%',
                'fill': _htOption.colorLight
            });
            svg.setAttributeNS("http://www.w3.org/2000/xmlns/", "xmlns:xlink", "http://www.w3.org/1999/xlink");
            _el.appendChild(svg);

            svg.appendChild(makeSVG("rect", {
                "fill": _htOption.colorLight,
                "width": "100%",
                "height": "100%"
            }));
            svg.appendChild(makeSVG("rect", {
                "fill": _htOption.colorDark,
                "width": "1",
                "height": "1",
                "id": "template"
            }));

            for (var row = 0; row < nCount; row++) {
                for (var col = 0; col < nCount; col++) {
                    if (oQRCode.isDark(row, col)) {
                        var child = makeSVG("use", {
                            "x": String(col),
                            "y": String(row)
                        });
                        child.setAttributeNS("http://www.w3.org/1999/xlink", "href", "#template")
                        svg.appendChild(child);
                    }
                }
            }
        };
        Drawing.prototype.clear = function() {
            while (this._el.hasChildNodes())
                this._el.removeChild(this._el.lastChild);
        };
        return Drawing;
    })();

    var useSVG = document.documentElement.tagName.toLowerCase() === "svg";

    // Drawing in DOM by using Table tag
    var Drawing = useSVG ? svgDrawer : !_isSupportCanvas() ? (function() {
        var Drawing = function(el, htOption) {
            this._el = el;
            this._htOption = htOption;
        };

        /**
         * Draw the QRCode
         * 
         * @param {QRCode} oQRCode
         */
        Drawing.prototype.draw = function(oQRCode) {
            var _htOption = this._htOption;
            var _el = this._el;
            var nCount = oQRCode.getModuleCount();
            var nWidth = Math.floor(_htOption.width / nCount);
            var nHeight = Math.floor(_htOption.height / nCount);
            var aHTML = ['<table style="border:0;border-collapse:collapse;">'];

            for (var row = 0; row < nCount; row++) {
                aHTML.push('<tr>');

                for (var col = 0; col < nCount; col++) {
                    aHTML.push('<td style="border:0;border-collapse:collapse;padding:0;margin:0;width:' + nWidth + 'px;height:' + nHeight + 'px;background-color:' + (oQRCode.isDark(row, col) ? _htOption.colorDark : _htOption.colorLight) + ';"></td>');
                }

                aHTML.push('</tr>');
            }

            aHTML.push('</table>');
            _el.innerHTML = aHTML.join('');

            // Fix the margin values as real size.
            var elTable = _el.childNodes[0];
            var nLeftMarginTable = (_htOption.width - elTable.offsetWidth) / 2;
            var nTopMarginTable = (_htOption.height - elTable.offsetHeight) / 2;

            if (nLeftMarginTable > 0 && nTopMarginTable > 0) {
                elTable.style.margin = nTopMarginTable + "px " + nLeftMarginTable + "px";
            }
        };

        /**
         * Clear the QRCode
         */
        Drawing.prototype.clear = function() {
            this._el.innerHTML = '';
        };

        return Drawing;
    })() : (function() { // Drawing in Canvas
        function _onMakeImage() {
            this._elImage.src = this._elCanvas.toDataURL("image/png");
            setLocalVar("qrcode_uri", this._elCanvas.toDataURL("image/png"));
            this._elImage.style.display = "inline-block";
            this._elCanvas.style.display = "none";
        }

        // Android 2.1 bug workaround
        // http://code.google.com/p/android/issues/detail?id=5141
        if (this._android && this._android <= 2.1) {
            var factor = 1 / window.devicePixelRatio;
            var drawImage = CanvasRenderingContext2D.prototype.drawImage;
            CanvasRenderingContext2D.prototype.drawImage = function(image, sx, sy, sw, sh, dx, dy, dw, dh) {
                if (("nodeName" in image) && /img/i.test(image.nodeName)) {
                    for (var i = arguments.length - 1; i >= 1; i--) {
                        arguments[i] = arguments[i] * factor;
                    }
                } else if (typeof dw == "undefined") {
                    arguments[1] *= factor;
                    arguments[2] *= factor;
                    arguments[3] *= factor;
                    arguments[4] *= factor;
                }

                drawImage.apply(this, arguments);
            };
        }

        /**
         * Check whether the user's browser supports Data URI or not
         * 
         * @private
         * @param {Function} fSuccess Occurs if it supports Data URI
         * @param {Function} fFail Occurs if it doesn't support Data URI
         */
        function _safeSetDataURI(fSuccess, fFail) {
            var self = this;
            self._fFail = fFail;
            self._fSuccess = fSuccess;

            // Check it just once
            if (self._bSupportDataURI === null) {
                var el = document.createElement("img");
                var fOnError = function() {
                    self._bSupportDataURI = false;

                    if (self._fFail) {
                        self._fFail.call(self);
                    }
                };
                var fOnSuccess = function() {
                    self._bSupportDataURI = true;

                    if (self._fSuccess) {
                        self._fSuccess.call(self);
                    }
                };

                el.onabort = fOnError;
                el.onerror = fOnError;
                el.onload = fOnSuccess;
                el.src = "data:image/gif;base64,iVBORw0KGgoAAAANSUhEUgAAAAUAAAAFCAYAAACNbyblAAAAHElEQVQI12P4//8/w38GIAXDIBKE0DHxgljNBAAO9TXL0Y4OHwAAAABJRU5ErkJggg=="; // the Image contains 1px data.
                return;
            } else if (self._bSupportDataURI === true && self._fSuccess) {
                self._fSuccess.call(self);
            } else if (self._bSupportDataURI === false && self._fFail) {
                self._fFail.call(self);
            }
        };

        /**
         * Drawing QRCode by using canvas
         * 
         * @constructor
         * @param {HTMLElement} el
         * @param {Object} htOption QRCode Options 
         */
        var Drawing = function(el, htOption, dataURI) {
            this._bIsPainted = false;
            this._android = _getAndroid();

            this._htOption = htOption;
            this._elCanvas = document.createElement("canvas");
            this._elCanvas.width = htOption.width;
            this._elCanvas.height = htOption.height;
            el.appendChild(this._elCanvas);
            this._el = el;
            this._oContext = this._elCanvas.getContext("2d");
            this._bIsPainted = false;
            this._elImage = document.createElement("img");
            this._elImage.alt = "qrCodeGenerated";
            this._elImage.style.display = "none";
            this._el.appendChild(this._elImage);
            this._bSupportDataURI = null;

        };

        /**
         * Draw the QRCode
         * 
         * @param {QRCode} oQRCode 
         */
        Drawing.prototype.draw = function(oQRCode) {
            var _elImage = this._elImage;
            var _oContext = this._oContext;
            var _htOption = this._htOption;

            var nCount = oQRCode.getModuleCount();
            var nWidth = _htOption.width / nCount;
            var nHeight = _htOption.height / nCount;
            var nRoundedWidth = Math.round(nWidth);
            var nRoundedHeight = Math.round(nHeight);

            _elImage.style.display = "none";
            this.clear();

            for (var row = 0; row < nCount; row++) {
                for (var col = 0; col < nCount; col++) {
                    var bIsDark = oQRCode.isDark(row, col);
                    var nLeft = col * nWidth;
                    var nTop = row * nHeight;
                    _oContext.strokeStyle = bIsDark ? _htOption.colorDark : _htOption.colorLight;
                    _oContext.lineWidth = 1;
                    _oContext.fillStyle = bIsDark ? _htOption.colorDark : _htOption.colorLight;
                    _oContext.fillRect(nLeft, nTop, nWidth, nHeight);

                    // 안티 앨리어싱 방지 처리
                    _oContext.strokeRect(
                        Math.floor(nLeft) + 0.5,
                        Math.floor(nTop) + 0.5,
                        nRoundedWidth,
                        nRoundedHeight
                    );

                    _oContext.strokeRect(
                        Math.ceil(nLeft) - 0.5,
                        Math.ceil(nTop) - 0.5,
                        nRoundedWidth,
                        nRoundedHeight
                    );
                }
            }

            this._bIsPainted = true;
        };

        /**
         * Make the image from Canvas if the browser supports Data URI.
         */
        Drawing.prototype.makeImage = function() {
            if (this._bIsPainted) {
                _safeSetDataURI.call(this, _onMakeImage);
            }
        };

        /**
         * Return whether the QRCode is painted or not
         * 
         * @return {Boolean}
         */
        Drawing.prototype.isPainted = function() {
            return this._bIsPainted;
        };

        /**
         * Clear the QRCode
         */
        Drawing.prototype.clear = function() {
            this._oContext.clearRect(0, 0, this._elCanvas.width, this._elCanvas.height);
            this._bIsPainted = false;
        };

        /**
         * @private
         * @param {Number} nNumber
         */
        Drawing.prototype.round = function(nNumber) {
            if (!nNumber) {
                return nNumber;
            }

            return Math.floor(nNumber * 1000) / 1000;
        };

        return Drawing;
    })();

    /**
     * Get the type by string length
     * 
     * @private
     * @param {String} sText
     * @param {Number} nCorrectLevel
     * @return {Number} type
     */
    function _getTypeNumber(sText, nCorrectLevel) {
        var nType = 1;
        var length = _getUTF8Length(sText);

        for (var i = 0, len = QRCodeLimitLength.length; i <= len; i++) {
            var nLimit = 0;

            switch (nCorrectLevel) {
                case QRErrorCorrectLevel.L:
                    nLimit = QRCodeLimitLength[i][0];
                    break;
                case QRErrorCorrectLevel.M:
                    nLimit = QRCodeLimitLength[i][1];
                    break;
                case QRErrorCorrectLevel.Q:
                    nLimit = QRCodeLimitLength[i][2];
                    break;
                case QRErrorCorrectLevel.H:
                    nLimit = QRCodeLimitLength[i][3];
                    break;
            }

            if (length <= nLimit) {
                break;
            } else {
                nType++;
            }
        }

        if (nType > QRCodeLimitLength.length) {
            throw new Error("Too long data");
        }

        return nType;
    }

    function _getUTF8Length(sText) {
        var replacedText = encodeURI(sText).toString().replace(/\%[0-9a-fA-F]{2}/g, 'a');
        return replacedText.length + (replacedText.length != sText ? 3 : 0);
    }

    /**
     * @class QRCode
     * @constructor
     * @example 
     * new QRCode(document.getElementById("test"), "http://jindo.dev.naver.com/collie");
     *
     * @example
     * var oQRCode = new QRCode("test", {
     *    text : "http://naver.com",
     *    width : 128,
     *    height : 128
     * });
     * 
     * oQRCode.clear(); // Clear the QRCode.
     * oQRCode.makeCode("http://map.naver.com"); // Re-create the QRCode.
     *
     * @param {HTMLElement|String} el target element or 'id' attribute of element.
     * @param {Object|String} vOption
     * @param {String} vOption.text QRCode link data
     * @param {Number} [vOption.width=256]
     * @param {Number} [vOption.height=256]
     * @param {String} [vOption.colorDark="#000000"]
     * @param {String} [vOption.colorLight="#ffffff"]
     * @param {QRCode.CorrectLevel} [vOption.correctLevel=QRCode.CorrectLevel.H] [L|M|Q|H] 
     */
    QRCode = function(el, vOption) {
        this._htOption = {
            width: 256,
            height: 256,
            typeNumber: 4,
            colorDark: "#000000",
            colorLight: "#ffffff",
            correctLevel: QRErrorCorrectLevel.H
        };

        if (typeof vOption === 'string') {
            vOption = {
                text: vOption
            };
        }

        // Overwrites options
        if (vOption) {
            for (var i in vOption) {
                this._htOption[i] = vOption[i];
            }
        }

        if (typeof el == "string") {
            el = document.getElementById(el);
        }

        if (this._htOption.useSVG) {
            Drawing = svgDrawer;
        }

        this._android = _getAndroid();
        this._el = el;
        this._oQRCode = null;
        this._oDrawing = new Drawing(this._el, this._htOption);

        if (this._htOption.text) {
            this.makeCode(this._htOption.text);
        }
    };

    /**
     * Make the QRCode
     * 
     * @param {String} sText link data
     */
    QRCode.prototype.makeCode = function(sText) {
        this._oQRCode = new QRCodeModel(_getTypeNumber(sText, this._htOption.correctLevel), this._htOption.correctLevel);
        this._oQRCode.addData(sText);
        this._oQRCode.make();
        this._oDrawing.draw(this._oQRCode);
        this.makeImage();
    };

    /**
     * Make the Image from Canvas element
     * - It occurs automatically
     * - Android below 3 doesn't support Data-URI spec.
     * 
     * @private
     */
    QRCode.prototype.makeImage = function() {
        if (typeof this._oDrawing.makeImage == "function" && (!this._android || this._android >= 3)) {
            this._oDrawing.makeImage();
        }
    };

    /**
     * Clear the QRCode
     */
    QRCode.prototype.clear = function() {
        this._oDrawing.clear();
    };

    /**
     * @name QRCode.CorrectLevel
     */
    QRCode.CorrectLevel = QRErrorCorrectLevel;
})();
/* jshint ignore:end */

// FILE: ./assets/js/services/ControlePedidosService.js
function ControlePedidosService(restEngine) {

    this.generateOrCancelNF = function(orders, operationType) {
        return restEngine.post('myo#/controle_pedidos/generateOrCancelNF', {
            "requestType": "DataSet",
            "dataset": [orders, operationType]
        }, null, null, null, null, 1500000).then(function(response) {
            return response.dataset.GERA_NF_MYORDER;
        });
    };

    this.generateRequisicaoExtra = function(dataset) {
        return restEngine.post('myo#/controle_pedidos/generateRequisicaoExtra', {
            "requestType": "DataSet",
            "dataset": dataset
        }, null, null, null, null, 1500000);
    };

    this.getOrdersNotAllowed = function(ordersId) {
        return restEngine.post('myo#/crud/notafiscal_pedidoserv', {
            "requestType": "FilterData",
            "filter": [
                {
                    "name": "NRPEDIDOSERV",
                    "operator": "=",
                    "value": ordersId
                }
            ]
        }).then(function(response) {
            return response.dataset.notafiscal_pedidoserv.reduce(function(prev, order) {
                return prev.concat(order.NRPEDIDOSERV);
            }, []);
        });
    };

    this.changeBackStatus = function(dataset) {
        return restEngine.post('myo#/controle_pedidos/changeBackStatus', {
            "requestType": "DataSet",
            "dataset": dataset
        });
    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register('controlePedidosService', ControlePedidosService);
});

// FILE: ./assets/js/controllers/CentroDeCustoController.js
/*global Util, angular, Configuration*/
var CentroDeCustoController = function(ScreenService, General, templateManager, ZhUtil, maskEngine, MessageService) {

    var _filterRowWidgetMain = {};
    this.filterCliente = function(field) {
        field.widget.getField("NMRAZSOCCLIE").clearValue();
        field.widget.getField("NMRAZSOCCLIE").dataSourceFilter = [
            {
                name: "CDFILIAL",
                operator: "=",
                value: field.widget.currentRow.CDFILIAL || "T"
			}
		];
        field.widget.getField("NMRAZSOCCLIE").reload();
    };

    this.prepareFilterToApply = function(args) {
        var filterRow = args.e.preference.FILTER_ROW;
        args.e.preference.CUSTOM_FILTER_ROW = {
            CDFILIAL: filterRow.CDFILIAL || "T",
            CDCLIENTE: filterRow.CDCLIENTE || null
        };
    };

    this.setStripeValue = (args) => {
        let stripeWidget = templateManager.container.getWidget('widgetStripe');
        let widgetGrid = templateManager.container.getWidget('widgetMain');
        const {
            CDCLIENTE,
            CDFILIAL,
            NMFILIAL,
            NMRAZSOCCLIE,
            NRINSJURCLIE
        } = widgetGrid.appliedFilter.FILTER_ROW;
        _filterRowWidgetMain = widgetGrid.appliedFilter.FILTER_ROW;
        let mask = {
            type: "CPFCNPJ"
        };
        let nrInsJurClientMask = Util.applyMask(NRINSJURCLIE, mask, maskEngine, ZhUtil);

        stripeWidget.currentRow.CDFILIAL = CDFILIAL + " - " + NMFILIAL;
        stripeWidget.currentRow.CDCLIENTE = nrInsJurClientMask + " - " + NMRAZSOCCLIE;
    };

    this.setParamsNewCode = function(widget) {
        var paramsNewCode = widget.paramsNewCode;
        widget.currentRow.__using_unit = paramsNewCode.__using_unit || false;
        widget.currentRow.__new_code_pk = paramsNewCode.__new_code_pk;
        widget.currentRow.__table_pk = paramsNewCode.__table_pk;
        widget.currentRow.__new_code_length = paramsNewCode.__new_code_length || 1;
        widget.currentRow.__cast_number = paramsNewCode.__cast_number || false;
    };

    this.editCentroDeCusto = function(centroWidget) {
        General.edit(centroWidget);
    };

    this.cancelAddEdit = function(args) {
        General.backOrCancel(args);
    };

    this.saveRowCentro = function(centroWidget) {
        if (centroWidget.isValid()) {
            this.setParamsNewCode(centroWidget);
            var regExp = /^[a-z0-9]+$/i;
            var regExpCaracter = /^[a-z\u00C0-\u00FF0-9\s]+$/i;
            var cdCentCusto = centroWidget.currentRow.CDCCUSCLIEFILIAL;
            var NmCentCusto = centroWidget.currentRow.NMCCUSCLIEFILIAL;

            if (regExp.test(cdCentCusto) && regExpCaracter.test(NmCentCusto)) {
                centroWidget.currentRow.CDFILIAL = _filterRowWidgetMain.CDFILIAL;
                centroWidget.currentRow.CDCLIENTE = _filterRowWidgetMain.CDCLIENTE;

                General.callRestEngine("myo#/centro_de_custo/save", "DataSet", [centroWidget.currentRow]).then(function(resolve) {
                    ScreenService.notificationMessage("Salvo com sucesso.", "success");
                    ScreenService.goBack();
                    centroWidget.reload();
                }, function(reject) {
                    ScreenService.showMessage("Registro já existente.");
                }).catch(this.handleError);
            } else {
                MessageService.showMessageByCode("VALIDATION_CHAR", {
                    cdCentCusto,
                    NmCentCusto
                });
            }
        }
    };

    function getValueByField(widget, fieldName, checkFieldName) {
        var value = "";
        if (widget.getField(checkFieldName).checked) {
            value = "T";
        } else {
            var fieldValue = widget.currentRow[fieldName];
            if (!Util.isEmptyOrBlank(fieldValue) && fieldValue.length) {
                value = fieldValue.join('_');
            }
        }
        return value;
    }

    this.updateCliente = function(widget) {
        var filter = {
            CDFILIAL: getValueByField(widget, "NMFILIAL", "TFILIAL")
        };
        General.onClickDependentField(widget.getField("CLIENTE"), filter);
    };

    this.clearUnidade = function(widget) {
        widget.currentRow.NMFILIAL = "";
        this.clearCliente(widget);
    };

    this.clearCliente = function(widget) {
        widget.currentRow.CLIENTE = "";
    };

    this.openPopupExport = function(widget) {
        var reportPopup = widget.container.getWidget('reportPopup');
        ScreenService.openPopup(reportPopup).then(function() {
            reportPopup.newRow();
            reportPopup.fields.forEach(function(field) {
                if (field.template.indexOf("multiple.html") !== -1) {
                    field.readOnly = false;
                    field.checked = false;
                } else {
                    field.checked = false;
                    General.toggleReadOnlySelect(field, reportPopup.getField(field.selectField));
                }
            });
            this.clearUnidade(reportPopup);
        }.bind(this));
    };

    function createExportData(widget, gridWidget) {
        var data = [];
        var row = angular.copy(widget.currentRow);
        row.paramsNewCode = gridWidget.paramsNewCode;
        gridWidget.dataSourceFilter.forEach(function(filter) {
            row[filter.name] = filter.value;
        });
        if (row.TFILIAL == 'S' || !Array.isArray(row.NMFILIAL)) {
            row.NMFILIAL = 'T';
        } else {
            row.NMFILIAL = row.NMFILIAL.join('|');
        }
        if (row.TCDCLIENTE == 'S' || !Array.isArray(row.CLIENTE)) {
            row.CLIENTE = 'T';
        } else {
            row.CLIENTE = row.CLIENTE.join('|');
        }
        data.push(gridWidget.getCheckedRows());
        data.push(row);
        return data;
    }

    this.export = function(widget) {
        var gridWidget = widget.container.getWidget('widgetMain');
        General.callRestEngine('myo#/export_centro_custo', 'DataSet', createExportData(widget, gridWidget)).then(function(success) {
            ScreenService.showMessage(ScreenService.i18n("Exportação realizada com sucesso"), 'success');
            ScreenService.closePopup();
            gridWidget.unSelectAll();
            gridWidget.reload().then(function() {
                gridWidget.activate();
            });
        }, General.callbackError);
    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register('CentroDeCustoController', CentroDeCustoController);
});

// FILE: ./assets/js/controllers/ControlePedidosController.js
var ControlePedidosController = function(ScreenService, General, controlePedidosService, ZHPromise, templateManager, MessageService) {

    var nrpedidoserv, nritempedidoserv;

    var STATUS_FINALIZADO = 'FINALIZADO';

    this.getFilterValue = function() {
        return nrpedidoserv;
    };

    this.getPratoFilterValue = function() {
        return nritempedidoserv;
    };

    this.WidgetAfterMoveRowServico = function() {
        nrpedidoserv = templateManager.container.getWidget('widgetOrderGrid').currentRow.NRPEDIDOSERV;
    };

    this.WidgetAfterMoveRowPrato = function() {
        nritempedidoserv = templateManager.container.getWidget('orderServicesTab').currentRow.NRITEMPEDIDOSERV;
    };

    this.backToWidgetOrderGrid = function(widget) {
        var widgetOrderGrid = widget.container.getWidget("widgetOrderGrid");
        ScreenService.closeSwipe();
        widgetOrderGrid.activate();
    };

    function getWhichFinalizadoOrdersIdCannotBeProgress(orders) {
        var ordersId = orders.reduce(function(prev, order) {
            return prev.concat(order.NRPEDIDOSERV);
        }, []).join("_");

        return controlePedidosService.getOrdersNotAllowed(ordersId);
    }

    var nextStatusByDirection = ["APROVADO", "EMPRODUCAO", "FINALIZADO"];

    function getNextStatus(order, direction) {
        var length = nextStatusByDirection.length;
        var position = nextStatusByDirection.indexOf(order.IDSTATUS);
        if (position >= 0) {
            var nextPosition = position + direction;
            if (nextPosition >= 0 && nextPosition < length) {
                return nextStatusByDirection[nextPosition];
            } else {
                return order.IDSTATUS;
            }
        }
    }

    function createMessageForDeniedOrders(deniedOrders) {
        var ordersId = deniedOrders.reduce(function(prev, orders) {
            return prev.concat(orders);
        }, []).join(", ");
        return "Os seguintes pedidos não puderam ser avançados pois a NF está pendente: " + ordersId;
    }

    this.changeStatus = function(widget, nextOrBack) {
        var ordersToChangeStatus = widget.getCheckedRows();

        var ordersWithStatusFinalizado = [];

        ordersToChangeStatus.forEach(function(order) {
            if (order.IDSTATUS === STATUS_FINALIZADO) {
                if (order.NRLANCTONF !== null) {
                    ordersWithStatusFinalizado.push(order);
                }
            } else {
                order.IDSTATUS = getNextStatus(order, nextOrBack);
                order.changed = true;
            }
        });

        if (ordersWithStatusFinalizado.length > 0) {
            getWhichFinalizadoOrdersIdCannotBeProgress(ordersWithStatusFinalizado)
                .then(function(deniedOrders) {
                    ordersWithStatusFinalizado.forEach(function eachOrder(order) {
                        if (deniedOrders.find(function(deniedOrder) {
                                return deniedOrder == order.NRPEDIDOSERV;
                            }) === undefined) {
                            order.IDSTATUS = getNextStatus(order, nextOrBack);
                            order.changed = true;
                        }
                    });
                    var deniedMsg = '';
                    if (deniedOrders.length > 0) {
                        deniedMsg = createMessageForDeniedOrders(deniedOrders);
                    }
                    widget.dataSource.save();
                    widget.dataSource.save().then(function() {
                        widget.unSelectAll();
                        MessageService.successNotificationByCode('DENIED_MSG', {
                            "deniedMsg": deniedMsg
                        });
                    });
                });
        } else {
            widget.dataSource.save().then(function() {
                widget.unSelectAll();
                ScreenService.showMessage("Processo concluído.", "success");
            });
        }

    };

    this.validaReport = function(widget) {
        var reportName = widget.currentRow.REPORT_NAME;
        var currentRow = widget.currentRow;
        if (widget.isValid() && validaData(currentRow)) {
            let reportFormats = [
                {
                    name: "pdf",
                    value: "PDF"
                },
                {
                    name: "xlsx",
                    value: "XLSX"
                }
            ];
            General.openReportPopup(widget, this.abreRelatorio, [widget, reportName, currentRow], reportFormats);
        }
    };

    this.abreRelatorio = function(widget, reportName, currentRow) {
        General.openReport('myo#/controle_pedidos', reportName, currentRow);
    };

    function validaData(currentRow) {
        var dataInicial = currentRow.DTINICIAL ? moment(currentRow.DTINICIAL, 'DD/MM/YYYY') : false;
        var dataFinal = currentRow.DTFINAL ? moment(currentRow.DTFINAL, 'DD/MM/YYYY') : false;
        var mensagem = "A data inicial deve ser menor ou igual a data final.";
        if (dataInicial && dataFinal && dataFinal.isBefore(dataInicial)) {
            ScreenService.showMessage(mensagem, "error");
            return false;
        }
        return true;
    }

    this.clearUnit = function(widget) {
        widget.currentRow.NMFILIAL = "";
        this.clearService(widget);
    };

    this.clearService = function(widget) {
        widget.currentRow.NMSERVICO = "";
    };

    this.updateServices = function(widget) {
        var filter = {
            CDGRUPFILI: widget.currentRow.CDGRUPFILI || 'T',
            CDFILIAL: getValueByField(widget, "NMFILIAL", "TFILIAL"),
            CDGRUPSERV: getValueByField(widget, "NMGRUPSERV", "TGRUPSERV")
        };
        General.onClickDependentField(widget.getField("NMSERVICO"), filter);
    };

    this.updateCliente = function(widget) {
        var filter = {
            CDFILIAL: getValueByField(widget, "NMFILIAL", "TFILIAL")
        };
        General.onClickDependentField(widget.getField("CLIENT"), filter);
    };

    this.updateCentroCusto = function(widget) {
        var filter = {
            CDFILIAL: getValueByField(widget, "NMFILIAL", "TFILIAL"),
            CDCLIENTE: getValueByField(widget, "CLIENT", "TCDCLIENTE")
        };
        General.onClickDependentField(widget.getField("CENTROCUSTO"), filter);
    };

    function getValueByField(widget, fieldName, checkFieldName) {
        var value = "";
        if (widget.getField(checkFieldName).checked) {
            value = "T";
        } else {
            var fieldValue = widget.currentRow[fieldName];
            if (!Util.isEmptyOrBlank(fieldValue) && fieldValue.length) {
                value = fieldValue.join('_');
            }
        }
        return value;
    }

    this.updateUnit = function(field) {
        var cRow = field.widget.currentRow;

        var filter = {
            CDGRUPFILI: cRow.NMGRUPFILI ? cRow.CDGRUPFILI : 'T'
        };
        General.onClickDependentField(field, filter);
    };

    function getFilterCliente(ordersToHandleNF) {
        return [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": ordersToHandleNF[0].CDFILIAL
            },
            {
                "name": "CDCLIENTE",
                "operator": "IN",
                "value": ordersToHandleNF.map(function(row) {
                    return row.CDCLIENTE;
                })
            }
        ];
    }

    function checkIfSelectedOrdersAreValid(ordersToHandleNF, operationType) {
        var deferred = ZHPromise.defer();

        General.callRestEngine("myo#/crud/clienfilial_cliente", "FilterData", getFilterCliente(ordersToHandleNF)).then(function(callback) {
            if (callback.dataset && callback.dataset.clienfilial_cliente && callback.dataset.clienfilial_cliente.length) {
                var cliente = callback.dataset.clienfilial_cliente;
                var msg;
                var error = 0;
                var errorClientes = "";
                cliente.forEach(function(row) {
                    if (row.IDAPURAPEDMYORDER === 'S') {
                        if (error > 0) {
                            errorClientes += ", ";
                        }
                        errorClientes += row.NMRAZSOCCLIE;
                        error++;
                    }
                });
                if (error) {
                    if (error == 1) {
                        msg = "CLIENTE_N_PRAD_PED";
                    } else {
                        msg += "CLIENTES_N_PRAD_PED";
                    }
                    MessageService.showMessageByCode(msg, {
                        "errorClientes": errorClientes
                    });
                    deferred.reject();
                } else {
                    if (operationType === 'G') {
                        ordersToHandleNF.find(function(order) {
                            if (order.IDSTATUS !== STATUS_FINALIZADO) {
                                ScreenService.showMessage(ScreenService.i18n("Só é permitido gerar NFs de pedidos com status FINALIZADO."), "error");
                                deferred.reject();
                                return deferred.promise;
                            } else if (order.NRLANCTONF !== null) {
                                ScreenService.showMessage(ScreenService.i18n("Existe um pedido selecionado já com NF gerada."), "error");
                                deferred.reject();
                                return deferred.promise;
                            }
                        });
                        deferred.resolve();
                    } else {
                        ordersToHandleNF.find(function(order) {
                            if (order.NRLANCTONF === null) {
                                ScreenService.showMessage(ScreenService.i18n("Existe um pedido selecionado sem NF gerada para ser cancelada."), "error");
                                deferred.reject();
                                return deferred.promise;
                            } else {
                                deferred.resolve();
                            }
                        });
                    }
                }
            }
        }, General.callbackError);
        return deferred.promise;
    }

    function getMessage(message) {
        var msg;
        switch (message) {
            case 'OK':
                msg = 'Operação executada com sucesso!';
                break;
            case 'ERR_NCANCENT':
                msg = 'Operação bloqueada. Existe duplicata lançada para nota fiscal.';
                break;
            case 'ERR_NFAPROV':
                msg = 'Operação bloqueada. Existe nota fiscal já aprovada/emitida';
                break;
            case 'ERR_MS03':
                msg = 'Operação bloqueada. Existe boleto baixado para nota fiscal.';
                break;
            case 'ERR_MS04':
                msg = 'Operação bloqueada. Existe conta parcelada para nota fiscal.';
                break;
            case 'ERR_MS06':
                msg = 'Operação bloqueada. Existe ajuste lançado em boleto de nota.';
                break;
            case 'ERR_MS07':
                msg = 'Operação bloqueada. Existe boleto emitido em nota fiscal.';
                break;
            case 'ERR_MS08':
                msg = 'Operação bloqueada. Existe CNAB lançado em boleto de nota fiscal.';
                break;
            case 'ERR_MS09':
                msg = 'Operação bloqueada. Existe associação de boletos em nota fiscal.';
                break;
            case 'ERR_MS11':
                msg = 'Operação bloqueada. Existe crédito de boleto utilizado em nota fiscal.';
                break;
            default:
                msg = message;
                break;
        }

        return msg;
    }

    function prepareMessage(ids, orderResponseMsg) {
        return ScreenService.i18n("Pedido(s): ") + ids.join(', ') + " - " + getMessage(orderResponseMsg.split(';')[0]);
    }

    this.generateOrCancelNF = function(widget, operationType) {
        var ordersToHandleNF = widget.getCheckedRows();
        var operationTypeRow = {
            OPERACAO: operationType
        };
        var dtFaturaPopup = widget.container.getWidget('DTFATURA');
        var msg = '';

        checkIfSelectedOrdersAreValid(ordersToHandleNF, operationType).then(function(success) {
            if (operationType === 'G') {
                dtFaturaPopup.newRow();
                ScreenService.openPopup(dtFaturaPopup);
            } else {
                controlePedidosService.generateOrCancelNF(ordersToHandleNF, operationTypeRow).then(function(response) {
                    response.forEach(function(orderResponse) {
                        msg += prepareMessage(orderResponse.ID, orderResponse.P_RETORNO) + "<br>";
                    });
                    widget.reload().then(function() {
                        widget.unSelectAll();
                        ScreenService.showMessage(msg);
                    });
                });
            }
        });
    };

    this.generateNF = function(widget) {
        if (widget.isValid()) {
            var dtFatura = widget.currentRow.DTFATURA;
            var tipooper = widget.currentRow.CDTIPOOPER;
            var ordersWidget = widget.container.getWidget('widgetOrderGrid');
            var rows = ordersWidget.getCheckedRows();
            var msg = '';

            rows.forEach(function(row) {
                row.DTFATURA = dtFatura;
                row.CDTIPOOPER = tipooper;
            });

            controlePedidosService.generateOrCancelNF(rows, {
                OPERACAO: 'G'
            }).then(function(response) {
                response.forEach(function(orderResponse) {
                    msg += prepareMessage(orderResponse.ID, orderResponse.P_RETORNO) + "<br>";
                });
                ordersWidget.reload().then(function() {
                    ordersWidget.unSelectAll();
                    ScreenService.showMessage(msg).then(function() {
                        ScreenService.closePopup();
                    });
                });
            });
        }
    };

    this.clearAutocompleteField = function(widget) {

        widget.currentRow.CDGRUPFILI = '';
        widget.currentRow.NMGRUPFILI = '';
        widget.currentRow.CDGRUPSERV = '';
        widget.currentRow.NMGRUPSERV = '';
    };

    this.openReportPopup = function(widget) {
        ScreenService.openPopup(widget.container.getWidget('ReportPopup')).then(function() {
            this.clearAutocompleteField(widget.container.getWidget('ReportPopup'));
            this.clearUnit(widget.container.getWidget('ReportPopup'));
        }.bind(this));
    };

    //Rotina de validação da Action Retornar Status
    this.changeBackStatus = function(widget) {
        let formatedStrings;

        controlePedidosService.changeBackStatus(widget.getCheckedRows()).then(function(callback) {
            widget.unSelectAll();
            widget.reload().then(function() {
                if (callback.dataset.error && callback.dataset.error.length) {
                    callback.dataset.error.unshift(ScreenService.i18n("\t\tOperação bloqueada."));
                    MessageService.getMessageByCode("BACKSTATUS").then(message => {
                        callback.dataset.error.forEach(function(error) {
                            formatedStrings.push(ScreenService.i18n(message, error[0], error[1]));
                        });
                    }).then(function() {
                        General.showErrorMessages(widget, formatedStrings, ScreenService.i18n("Mudança de Status"));
                    });
                } else {
                    var type = "success";
                    var msg = ScreenService.i18n("Processo concluído.");
                    ScreenService.showMessage(msg, type).then(function() {
                        ScreenService.closePopup();
                    });
                }
            });
        }, General.callbackError);
    };

    this.generateRequisicaoExtra = function(widget) {
        if (widget.isValid()) {
            var dataset = widget.getCheckedRows();
            var invalidData = dataset.filter(function(row) {
                return row.IDSTATUS !== "EMPRODUCAO";
            });
            if (invalidData && invalidData.length) {
                ScreenService.showMessage(ScreenService.i18n("Operação bloqueada. Selecione apenas pedidos com status 'Em Produção'."), 'error');
            } else {
                controlePedidosService.generateRequisicaoExtra(dataset).then(function(response) {
                    widget.unSelectAll();
                    widget.reload().then(function() {
                        if (response.dataset && response.dataset.ListaErros && response.dataset.ListaErros.length) {
                            General.showErrorMessages(widget, response.dataset.ListaErros, ScreenService.i18n("Geração de Requisição Extra"));
                        } else {
                            ScreenService.showMessage(ScreenService.i18n("Geração concluída com sucesso."), "success").then(function() {
                                ScreenService.closePopup();
                            });
                        }
                    });
                });
            }
        }
    };

    this.setDataInterval = function(field) {
        var value = field.value();
        if (value && value.start && value.end) {
            field.widget.getField('DTINICIAL').setValue(value.start);
            field.widget.getField('DTFINAL').setValue(value.end);
        }
    };

    this.FieldOnClickCentCus = function(row) {
        var field = templateManager.container.getWidget('widgetOrderGrid').getField("NMCCUSCLIEFILIAL");
        field.dataSourceFilter[0].value = row.CDFILIAL;
    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register('controlePedidosController', ControlePedidosController);
});

// FILE: ./assets/js/controllers/RequisicaoPedidosController.js
var RequisicaoPedidosController = function(ScreenService, General, templateManager, maskEngine, ZhUtil, ZHPromise) {

    const self = this;
    const _messages = {
        libLowerThanZero: "A quantidade liberada deve ser maior ou igual à zero.",
        devLowerThanZero: "A quantidade devolvida deve ser maior ou igual à zero.",
        extLowerThanZero: "A quantidade extra deve ser maior ou igual à zero.",
        devGreaterThanSum: "A quantidade devolvida deve ser menor ou igual à soma das quantidades prevista e extra.",
        successRateio: "Rateio realizado com sucesso.",
        successSave: "Registro salvo com sucesso."
    };
    var _filterRowWidgetGridProdutos = {};

    this.filterAfterApply = function(args) {
        this.setFilterRow(args);
    };

    this.fieldOnChangeFiltro = function(args) {
        args.row.NRPEDIDOSERV = "";
    };

    this.setFilterRow = function(args) {
        var filterRow = args.e.preference.FILTER_ROW;
        _filterRowWidgetGridProdutos = filterRow;
        var stripe = args.owner.container.getWidget('stripe');
        stripe.getField('CDFILIAL').setValue(filterRow.CDFILIAL + ' - ' + filterRow.NMFILIAL);
        stripe.getField('NRPEDIDOSERV').setValue(filterRow.NRPEDIDOSERV);
        stripe.getField('DTENTREGA').setValue(filterRow.DTENTREGA);
    };

    this.openSwipeServico = function(widget) {
        var servicoWidget = widget.container.getWidget('widgetGridServicos');
        var servicoTabbedWidget = widget.container.getWidget('widgetTabbedServicos');
        if (validateInlineEditable(widget)) {
            if (!ScreenService.isSwipeOpen(0)) {
                ScreenService.openSwipe(servicoTabbedWidget).then(function() {
                    servicoWidget.activate();
                });
            } else {
                self.onEnterServico(servicoWidget);
            }
        }
    };

    this.onEnterServico = function(widget) {
        var produtoWidget = widget.container.getWidget('widgetGridProdutos');
        var filter = [
            {
                "name": "CDPRODUTO",
                "operator": "=",
                "value": produtoWidget.currentRow.CDPRODUTO
            },
            {
                "name": "NRPEDIDOSERV",
                "operator": "=",
                "value": produtoWidget.appliedFilter.FILTER_ROW.NRPEDIDOSERV
            }
        ];
        widget.dataSourceFilter = filter;
        widget.reload().then(function() {
            widget.activate();
        });
    };

    this.goBackProduto = function(widget) {
        var produtoWidget = widget.container.getWidget('widgetGridProdutos');
        ScreenService.closeSwipe().then(function() {
            produtoWidget.activate();
        });
    };

    this.saveInlineEditServico = function(widget) {
        if (validaQuantidades(widget, true, true, true) && widget.dataSource && widget.dataSource.data) {
            var servicosInlineEdit = widget.dataSource.data.filter(function(servico) {
                return servico.changed || Object.keys(servico).filter(function(key) {
                    return key.indexOf('_inline') !== -1;
                }).length;
            });
            if (servicosInlineEdit.length) {
                var produtoWidget = widget.container.getWidget('widgetGridProdutos');
                convertValuesInlineEdit(widget, servicosInlineEdit);
                General.callRestEngine("myo#/crud/prevreti/save", "DataSet", servicosInlineEdit).then(function() {
                    produtoWidget.reload().then(function() {
                        widget.reload().then(function() {
                            ScreenService.notificationMessage(_messages.successSave, "success");
                            widget.commitInlineEdit();
                            widget.list();
                            widget.activate();
                        });
                    });
                }, General.callbackError);
            }
        }
    };

    this.saveInlineEditProduto = function(widget) {
        if (validaQuantidades(widget, true, true, true) && widget.dataSource && widget.dataSource.data) {
            var produtosInlineEdit = widget.dataSource.data.filter(function(produto) {
                return produto.changed || Object.keys(produto).filter(function(key) {
                    return key.indexOf('_inline') !== -1;
                }).length;
            });
            if (produtosInlineEdit.length) {
                convertValuesInlineEdit(widget, produtosInlineEdit);
                General.callRestEngine("myo#/prevreti/saveRateio", "DataSet", produtosInlineEdit).then(function() {
                    widget.reload().then(function() {
                        ScreenService.notificationMessage(_messages.successRateio, "success");
                        widget.commitInlineEdit();
                        widget.list();
                        widget.activate();
                    });
                }, General.callbackError);
            }
        }
    };

    this.cancelInlineEdit = function(widget) {
        widget.rollbackInlineEdit();
        widget.list();
        widget.activate();
    };

    this.openReportPopup = function(widget) {
        var reportWidget = widget.container.getWidget('widgetPopupReport');
        ScreenService.openPopup(reportWidget).then(function() {
            reportWidget.newRow();
            reportWidget.currentRow.NMFILIAL = _filterRowWidgetGridProdutos.NMFILIAL;
            reportWidget.currentRow.CDFILIAL = _filterRowWidgetGridProdutos.CDFILIAL;
            reportWidget.activate();
        });
    };

    this.setDataInterval = function(field) {
        var value = field.value();
        if (value && value.start && value.end) {
            field.widget.getField('DTINICIAL').setValue(value.start);
            field.widget.getField('DTFINAL').setValue(value.end);
        }
    };

    this.validaReport = function(widget) {
        if (widget.isValid() && validaData(widget.currentRow)) {
            //Regex replace to all occurrences except the character '.'
            //Formatter mask for birt
            widget.currentRow.CDPICTPROD = getLocalVar("CDPICTPROD").replace(/[^(/\.|\ )]/g, "@");
            let reportFormats = [
                {
                    name: "pdf",
                    value: "PDF"
                },
                {
                    name: "xlsx",
                    value: "XLSX"
                }
            ];
            General.openReportPopup(widget, this.callReport, [widget, widget.currentRow.REPORT_NAME, widget.currentRow], reportFormats);
        }
    };

    function validaData(currentRow) {
        var dataInicial = currentRow.DTINICIAL ? moment(currentRow.DTINICIAL, 'DD/MM/YYYY') : false;
        var dataFinal = currentRow.DTFINAL ? moment(currentRow.DTFINAL, 'DD/MM/YYYY') : false;
        var mensagem = "A data inicial deve ser menor ou igual a data final.";
        if (dataInicial && dataFinal && dataFinal.isBefore(dataInicial)) {
            ScreenService.showMessage(mensagem, "error");
            return false;
        }
        return true;
    }

    this.callReport = function(widget, reportName, currentRow) {
        General.openReport('myo#/requisicao_pedidos_report', reportName, currentRow);
    };

    this.cancelReport = function(widget) {
        var produtoWidget = widget.container.getWidget('widgetGridProdutos');
        ScreenService.closePopup();
        produtoWidget.activate();
    };

    function convertValuesInlineEdit(widget, data) {
        var fieldsInlineEdit = widget.fields.filter(function(field) {
            return field.inlineEditable && field.inlineEditable === "float";
        }).map(function(field) {
            return field.name;
        });
        data.forEach(function(servico) {
            fieldsInlineEdit.forEach(function(name) {
                if (servico[name]) {
                    servico[name] = General.valueToFloat(servico[name]);
                }
            });
        });
    }

    function validateInlineEditable(widget) {
        return widget && !widget.editing && !Util.isClickOnEditableField(window.event);
    }

    var validaQuantidades = function(widget, lib, devol, extra) {
        var row = widget.currentRow;
        var QTPREVDEVOL = !Util.isUndefinedOrNull(row.QTPREVDEVOL_inline) ? General.valueToFloat(row.QTPREVDEVOL_inline) : General.valueToFloat(row.QTPREVDEVOL);
        var QTPREVEXTRA = !Util.isUndefinedOrNull(row.QTPREVEXTRA_inline) ? General.valueToFloat(row.QTPREVEXTRA_inline) : General.valueToFloat(row.QTPREVEXTRA);
        var QTPREVLIB = !Util.isUndefinedOrNull(row.QTPREVLIB_inline) ? General.valueToFloat(row.QTPREVLIB_inline) : General.valueToFloat(row.QTPREVLIB);
        var QTPREVRETI = General.valueToFloat(row.QTPREVRETI);
        if (QTPREVDEVOL > (QTPREVRETI + QTPREVEXTRA)) {
            ScreenService.showMessage(_messages.devGreaterThanSum, "error");
            validado = false;
        } else if (lib && QTPREVLIB < 0) {
            ScreenService.showMessage(_messages.libLowerThanZero, "error");
            validado = false;
        } else if (devol && QTPREVDEVOL < 0) {
            ScreenService.showMessage(_messages.devLowerThanZero, "error");
            validado = false;
        } else if (extra && QTPREVEXTRA < 0) {
            ScreenService.showMessage(_messages.extLowerThanZero, "error");
            validado = false;
        } else {
            validado = true;
        }
        return validado;
    };

    this.onChangeQtdeLib = function(widget) {
        var row = widget.currentRow;
        var QTPREVDEVOL = !Util.isUndefinedOrNull(row.QTPREVDEVOL_inline) ? General.valueToFloat(row.QTPREVDEVOL_inline) : General.valueToFloat(row.QTPREVDEVOL);
        var QTPREVEXTRA = !Util.isUndefinedOrNull(row.QTPREVEXTRA_inline) ? General.valueToFloat(row.QTPREVEXTRA_inline) : General.valueToFloat(row.QTPREVEXTRA);
        var QTPREVLIB = !Util.isUndefinedOrNull(row.QTPREVLIB_inline) ? General.valueToFloat(row.QTPREVLIB_inline) : General.valueToFloat(row.QTPREVLIB);
        var QTPREVRETI = General.valueToFloat(row.QTPREVRETI);
        if (validaQuantidades(widget, true, false, false)) {
            if (QTPREVRETI > 0) { // quando o Produto veio do Cálculo da Requisição
                var dif = fixFloat(QTPREVRETI - QTPREVLIB);
                if (dif > 0) {
                    row.changed_QTPREVDEVOL = row.changed_QTPREVDEVOL || QTPREVDEVOL;
                    QTPREVDEVOL = dif;
                    row.QTPREVDEVOL = QTPREVDEVOL;
                    row.QTPREVDEVOL_inline = row.QTPREVDEVOL;

                    row.changed_QTPREVEXTRA = row.changed_QTPREVEXTRA || QTPREVEXTRA;
                    QTPREVEXTRA = 0.0;
                    row.QTPREVEXTRA = QTPREVEXTRA;
                    row.QTPREVEXTRA_inline = row.QTPREVEXTRA;
                } else if (dif < 0) {
                    row.changed_QTPREVEXTRA = row.changed_QTPREVEXTRA || QTPREVEXTRA;
                    QTPREVEXTRA = dif * (-1);
                    row.QTPREVEXTRA = QTPREVEXTRA;
                    row.QTPREVEXTRA_inline = row.QTPREVEXTRA;

                    row.changed_QTPREVDEVOL = row.changed_QTPREVDEVOL || QTPREVDEVOL;
                    QTPREVDEVOL = 0.0;
                    row.QTPREVDEVOL = QTPREVDEVOL;
                    row.QTPREVDEVOL_inline = row.QTPREVDEVOL;

                } else if (dif === 0) {
                    row.changed_QTPREVEXTRA = row.changed_QTPREVEXTRA || QTPREVEXTRA;
                    QTPREVEXTRA = 0.0;
                    row.QTPREVEXTRA = QTPREVEXTRA;
                    row.QTPREVEXTRA_inline = row.QTPREVEXTRA;

                    row.changed_QTPREVDEVOL = row.changed_QTPREVDEVOL || QTPREVDEVOL;
                    QTPREVDEVOL = 0.0;
                    row.QTPREVDEVOL = QTPREVDEVOL;
                    row.QTPREVDEVOL_inline = row.QTPREVDEVOL;
                }
            } else { // quando está adicionando novo produto e a qtde Prevista é ZERO
                row.changed_QTPREVDEVOL = row.changed_QTPREVDEVOL || QTPREVDEVOL;
                QTPREVDEVOL = 0.0;
                row.QTPREVDEVOL = QTPREVDEVOL;
                row.QTPREVDEVOL_inline = row.QTPREVDEVOL;

                row.changed_QTPREVEXTRA = row.changed_QTPREVEXTRA || QTPREVEXTRA;
                row.QTPREVEXTRA = QTPREVLIB;
                row.QTPREVEXTRA_inline = row.QTPREVEXTRA;
            }
        }
    };

    this.onChangeQtdeDevol = function(widget) {
        var row = widget.currentRow;
        var QTPREVDEVOL = !Util.isUndefinedOrNull(row.QTPREVDEVOL_inline) ? General.valueToFloat(row.QTPREVDEVOL_inline) : General.valueToFloat(row.QTPREVDEVOL);
        var QTPREVEXTRA = !Util.isUndefinedOrNull(row.QTPREVEXTRA_inline) ? General.valueToFloat(row.QTPREVEXTRA_inline) : General.valueToFloat(row.QTPREVEXTRA);
        var QTPREVRETI = General.valueToFloat(row.QTPREVRETI);
        if (validaQuantidades(widget, false, true, false)) {
            row.changed_QTPREVLIB = row.changed_QTPREVLIB || row.QTPREVLIB;
            row.QTPREVLIB = fixFloat(QTPREVRETI + QTPREVEXTRA - QTPREVDEVOL);
            row.QTPREVLIB_inline = row.QTPREVLIB;
        }
    };

    this.onChangeQtdeExtra = function(widget) {
        var row = widget.currentRow;
        var QTPREVDEVOL = !Util.isUndefinedOrNull(row.QTPREVDEVOL_inline) ? General.valueToFloat(row.QTPREVDEVOL_inline) : General.valueToFloat(row.QTPREVDEVOL);
        var QTPREVEXTRA = !Util.isUndefinedOrNull(row.QTPREVEXTRA_inline) ? General.valueToFloat(row.QTPREVEXTRA_inline) : General.valueToFloat(row.QTPREVEXTRA);
        var QTPREVRETI = General.valueToFloat(row.QTPREVRETI);
        if (validaQuantidades(widget, false, false, true)) {

            row.changed_QTPREVLIB = row.changed_QTPREVLIB || row.QTPREVLIB;
            row.QTPREVLIB = fixFloat(QTPREVRETI + QTPREVEXTRA - QTPREVDEVOL);
            row.QTPREVLIB_inline = row.QTPREVLIB;

        }
    };

    function fixFloat(number) {
        return General.valueToFloat(number.toFixed(5));
    }

    this.onClickDependentFieldPedido = function(field) {
        General.onClickDependentField(field, {
            CDFILIAL: field.widget.currentRow.CDFILIAL ? field.widget.currentRow.CDFILIAL : getLocalVar('CDFILIAL'),
            DTENTREGA: field.widget.currentRow.DTENTREGA ? moment(field.widget.currentRow.DTENTREGA, "DD/MM/YYYY").format("DD/MM/YYYY") : "",
            NRORG: getLocalVar('NRORG')
        });
    };

    this.beforeMoveRowServicos = function(args) {
        var widget = args.owner;

        if (widget.editing && !validaQuantidades(widget, true, true, true)) {
            args.e.preventDefault();
        }
    };

    this.beforeMoveRowProdutos = function(args) {
        var widget = args.owner;

        if (widget.editing && !validaQuantidades(widget, true, true, true)) {
            args.e.preventDefault();
        }
    };

    this.onChangeReportUnidade = function() {
        var reportWidget = templateManager.container.getWidget("widgetPopupReport");
        var servicoField = reportWidget.getField("NMSERVICO");
        servicoField.clearValue();
    };

    this.onClickReportServico = function() {
        var reportWidget = templateManager.container.getWidget("widgetPopupReport");
        var servicoField = reportWidget.getField("NMSERVICO");
        var cdFilial = reportWidget.currentRow.CDFILIAL;
        if (Util.isEmptyOrBlank(cdFilial)) {
            servicoField.dataSource.data = [];
        } else {
            var filter = [
                {
                    "name": "CDFILIAL",
                    "operator": "=",
                    "value": cdFilial
                },
                {
                    "name": [
                        {
                            "name": "CDSERVFILI",
                            "direction": "ASC"
                        }
                    ],
                    "operator": "ORDER_BY"
                }
            ];
            servicoField.dataSourceFilter = filter;
            servicoField.reload();
        }
    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register('RequisicaoPedidosController', RequisicaoPedidosController);
});

// FILE: ./assets/js/controllers/UsuariosMyOrder_Controller.js
function UsuariosMyOrder_Controller(ScreenService, templateManager, General) {

    var self = this;

    var NrUsuarioApp = null;
    var CdFilial = null;

    this.setUsuario = function(widget) {
        NrUsuarioApp = widget.currentRow.NRUSUARIOAPP;
    };

    this.getUsuario = function() {
        return NrUsuarioApp;
    };

    this.setFilial = function(field) {
        if (field.value() === '')
            CdFilial = 'T';
        else
            CdFilial = field.value().join("_");

    };

    this.getFilial = function() {
        if (!CdFilial) CdFilial = 'T';
        return CdFilial;
    };

    this.newRegister = function(widget) {
        widget.newRow();
        widget.container.getWidget('UNIDADECLIENTE').isVisible = false;
    };

    this.registerUser = function(widget) {
        if (widget.isValid()) {
            var rg = /[a-z0-9!#$%&'*+/=?^_`{|}~-]+(?:\.[a-z0-9!#$%&'*+/=?^_`{|}~-]+)*@(?:[a-z0-9](?:[a-z0-9-]*[a-z0-9])?\.)+[a-z0-9](?:[a-z0-9-]*[a-z0-9])?/g;
            if (rg.test(widget.currentRow.DSEMAILUSUARIOAPP)) {
                if (widget.currentRow.__is_new === true) {
                    var uniCliWidget = templateManager.container.getWidget('UNIDADECLIENTE');
                    uniCliWidget.isVisible = true;
                    widget.openTab('UNIDADECLIENTE');
                    ScreenService.showMessage("Para finalizar o cadastro é preciso associar uma ou mais Unidades/Clientes ao novo usuário").then(function() {
                        uniCliWidget.newRow();
                        uniCliWidget.__new_user = true;
                        self.limpaTela(uniCliWidget);
                    });
                }
            } else {
                ScreenService.showMessage("Email Inválido!");
            }
        }
    };

    this.edit = function(widget) {
        widget.getField('NMFILIAL_INSERCAO').isVisible = false;
        widget.getField('TFILIAL').isVisible = false;
        widget.getField('NMRAZSOCCLIE_INSERCAO').isVisible = false;
        widget.getField('TCDCLIENTE').isVisible = false;
        widget.edit();
    };

    this.salvaUnidadeCliente = function(widget) {
        var request, unidadeCliente;
        var papel = {
            "SOLICITA": widget.getField('SOLICITA').value() === 'S' ? '1' : 'N',
            "APROVA": widget.getField('APROVA').value() === 'S' ? '2' : 'N'
        };

        if (widget.isValid()) {
            if (widget.__new_user === true) {
                //Dados USUARIOAPP
                var usuario = {
                    "DSEMAILUSUARIOAPP": widget.getParent().currentRow.DSEMAILUSUARIOAPP,
                    "DSPRIMEIRONOME": widget.getParent().currentRow.DSPRIMEIRONOME,
                    "DSULTIMONOME": widget.getParent().currentRow.DSULTIMONOME,
                    "DSTELEFONE": widget.getParent().currentRow.DSTELEFONE
                };

                unidadeCliente = {
                    "CDFILIAL": widget.getField('NMFILIAL_INSERCAO').value() ? widget.getField('NMFILIAL_INSERCAO').value().join('_') : 'T',
                    "CDCLIENTE": widget.getField('NMRAZSOCCLIE_INSERCAO').value() ? widget.getField('NMRAZSOCCLIE_INSERCAO').value().join('_') : 'T'
                };

                request = {
                    route: 'myo#/salvaNovoUsuarioMyOrder',
                    type: 'Row',
                    params: [{
                        "usuario": usuario,
                        "unidadeCliente": unidadeCliente,
                        "papel": papel
                    }]
                };

                General.callRestEngine(request.route, request.type, request.params).then(function() {
                    widget.getField('NMFILIAL_INSERCAO').isVisible = true;
                    widget.getField('TFILIAL').isVisible = true;
                    widget.getField('NMRAZSOCCLIE_INSERCAO').isVisible = true;
                    widget.getField('TCDCLIENTE').isVisible = true;

                    widget.getParent().getField('DSPRIMEIRONOME').readOnly = true;
                    widget.getParent().getField('DSULTIMONOME').readOnly = true;
                    widget.getParent().getField('DSTELEFONE').readOnly = true;

                    ScreenService.goBack();
                    ScreenService.closeSwipe(0);
                    widget.getParent().reload();
                    widget.__new_user = false;
                });
            } else if (widget.currentRow.__is_new === true) {
                unidadeCliente = {
                    "CDFILIAL": widget.getField('NMFILIAL_INSERCAO').value() ? widget.getField('NMFILIAL_INSERCAO').value().join('_') : 'T',
                    "CDCLIENTE": widget.getField('NMRAZSOCCLIE_INSERCAO').value() ? widget.getField('NMRAZSOCCLIE_INSERCAO').value().join('_') : 'T'
                };

                request = {
                    route: 'myo#/incluiCliFilMyOrder',
                    type: 'Row',
                    params: [{
                        "usuario": widget.getParent().currentRow.NRUSUARIOAPP,
                        "unidadeCliente": unidadeCliente,
                        "papel": papel
                    }]
                };
                General.callRestEngine(request.route, request.type, request.params).then(function() {
                    widget.getField('NMFILIAL_INSERCAO').isVisible = true;
                    widget.getField('TFILIAL').isVisible = true;
                    widget.getField('NMRAZSOCCLIE_INSERCAO').isVisible = true;
                    widget.getField('TCDCLIENTE').isVisible = true;

                    widget.getParent().getField('DSPRIMEIRONOME').readOnly = true;
                    widget.getParent().getField('DSULTIMONOME').readOnly = true;
                    widget.getParent().getField('DSTELEFONE').readOnly = true;

                    ScreenService.goBack();
                    widget.reload();
                    widget.__new_user = false;
                });
            } else {
                request = {
                    route: 'myo#/editaUsuarioMyOrder',
                    type: 'Row',
                    params: [widget.currentRow, papel]
                };
                General.callRestEngine(request.route, request.type, request.params).then(function() {
                    widget.getField('NMFILIAL_INSERCAO').isVisible = true;
                    widget.getField('TFILIAL').isVisible = true;
                    widget.getField('NMRAZSOCCLIE_INSERCAO').isVisible = true;
                    widget.getField('TCDCLIENTE').isVisible = true;

                    widget.getParent().getField('DSPRIMEIRONOME').readOnly = true;
                    widget.getParent().getField('DSULTIMONOME').readOnly = true;
                    widget.getParent().getField('DSTELEFONE').readOnly = true;

                    ScreenService.goBack();
                    widget.__new_user = false;
                });
            }
        }
    };

    this.limpaCliente = function(widget) {
        this.setFilial(widget.getField('NMFILIAL_INSERCAO'));
        widget.getField('NMRAZSOCCLIE_INSERCAO').clearValue();
        widget.getField('NMRAZSOCCLIE_INSERCAO').reload();
    };

    this.cancel = function(widget) {
        if (widget.__new_user === true) {
            ScreenService.closeSwipe(0).then(function() {
                widget.__new_user = false;

                widget.getParent().activate();
                widget.getParent().dataSource.data.pop();
                widget.getParent().currentRow = {};
            });
        }

        widget.getField('NMFILIAL_INSERCAO').isVisible = true;
        widget.getField('TFILIAL').isVisible = true;
        widget.getField('NMRAZSOCCLIE_INSERCAO').isVisible = true;
        widget.getField('TCDCLIENTE').isVisible = true;

        widget.getParent().getField('DSPRIMEIRONOME').readOnly = true;
        widget.getParent().getField('DSULTIMONOME').readOnly = true;
        widget.getParent().getField('DSTELEFONE').readOnly = true;
    };

    this.validaUsuarioPorEmail = function(field) {
        if (typeof field.value() === "string") {
            field.value((field.value().toLowerCase()));
        }
        var rg = /[a-z0-9!#$%&'*+/=?^_`{|}~-]+(?:\.[a-z0-9!#$%&'*+/=?^_`{|}~-]+)*@(?:[a-z0-9](?:[a-z0-9-]*[a-z0-9])?\.)+[a-z0-9](?:[a-z0-9-]*[a-z0-9])?/g;
        if (rg.test(field.value())) {
            var request = {
                route: 'myo#/crud/usuarioapp',
                type: 'FilterData',
                params: [{
                        name: 'DSEMAILUSUARIOAPP',
                        operator: '=',
                        value: field.value()
                    },
                    {
                        name: 'NRORG',
                        operator: '=',
                        value: getLocalVar('NRORG')
                    }
                ]
            };

            General.callRestEngine(request.route, request.type, request.params).then(function(callback) {
                var dados = callback.dataset.usuarioapp;
                if (dados.length > 0) {
                    field.widget.getField('NRUSUARIOAPP').readOnly = true;
                    field.widget.currentRow.NRUSUARIOAPP = callback.dataset.usuarioapp[0].NRUSUARIOAPP;
                    field.widget.getField('DSPRIMEIRONOME').readOnly = true;
                    field.widget.currentRow.DSPRIMEIRONOME = callback.dataset.usuarioapp[0].DSPRIMEIRONOME;
                    field.widget.getField('DSULTIMONOME').readOnly = true;
                    field.widget.currentRow.DSULTIMONOME = callback.dataset.usuarioapp[0].DSULTIMONOME;
                    field.widget.getField('DSTELEFONE').readOnly = true;
                    field.widget.currentRow.DSTELEFONE = callback.dataset.usuarioapp[0].DSTELEFONE;
                } else {
                    field.widget.getField('NRUSUARIOAPP').readOnly = false;
                    field.widget.getField('NRUSUARIOAPP').setValue('');
                    field.widget.getField('DSPRIMEIRONOME').readOnly = false;
                    field.widget.getField('DSPRIMEIRONOME').setValue('');
                    field.widget.getField('DSULTIMONOME').readOnly = false;
                    field.widget.getField('DSULTIMONOME').setValue('');
                    field.widget.getField('DSTELEFONE').readOnly = false;
                    field.widget.getField('DSTELEFONE').setValue('');
                }
            });
        } else {
            ScreenService.showMessage("Email Inválido!");
            field.widget.getField('NRUSUARIOAPP').readOnly = true;
            field.widget.getField('NRUSUARIOAPP').setValue('');
            field.widget.getField('DSPRIMEIRONOME').readOnly = true;
            field.widget.getField('DSPRIMEIRONOME').setValue('');
            field.widget.getField('DSULTIMONOME').readOnly = true;
            field.widget.getField('DSULTIMONOME').setValue('');
            field.widget.getField('DSTELEFONE').readOnly = true;
            field.widget.getField('DSTELEFONE').setValue('');
        }
    };

    this.cancelRegister = function(args) {
        General.backOrCancel(args);
        args.owner.widget.getField('DSPRIMEIRONOME').readOnly = true;
        args.owner.widget.getField('DSULTIMONOME').readOnly = true;
        args.owner.widget.getField('DSTELEFONE').readOnly = true;

        args.owner.widget.container.getWidget('UNIDADECLIENTE').isVisible = true;
    };

    this.removeUser = function(widget) {
        ScreenService.confirmMessage(
            ScreenService.i18n(
                'Ao excluir o usuário, a relação com o MyOrder será excluída também. Deseja continuar?'
            ), 'question',
            function(success) {
                var request = {
                    route: 'myo#/excluiUsuarioMyOrder',
                    type: 'Row',
                    params: [widget.currentRow.NRUSUARIOAPP]
                };

                General.callRestEngine(request.route, request.type, request.params).then(function() {
                    ScreenService.goBack();
                    widget.reload();
                }, function(error) {
                    ScreenService.showMessage(error.data.error);
                });
            });

    };

    this.removeCliFil = function(widget) {
        ScreenService.confirmMessage(
            ScreenService.i18n(
                'O registro será excluído. Deseja continuar?'
            ), 'question',
            function(success) {
                var request = {
                    route: 'myo#/excluiCliFilUsuario',
                    type: 'Row',
                    params: [widget.currentRow.NRUSUARIOCLIFIL]
                };

                General.callRestEngine(request.route, request.type, request.params).then(function() {
                    ScreenService.goBack();
                    widget.reload();
                }, function(error) {
                    ScreenService.showMessage(error.data.error);
                });
            });

    };

    this.limpaTela = function(widget) {
        widget.getField('NMFILIAL_INSERCAO').validations = {};
        widget.getField('NMFILIAL_INSERCAO').clearValue();
        widget.getField('NMFILIAL_INSERCAO').readOnly = true;
        widget.getField('TFILIAL').checked = true;
        widget.getField('TFILIAL').setValue('S');

        widget.getField('NMRAZSOCCLIE_INSERCAO').validations = {};
        widget.getField('NMRAZSOCCLIE_INSERCAO').clearValue();
        widget.getField('NMRAZSOCCLIE_INSERCAO').readOnly = true;
        widget.getField('TCDCLIENTE').checked = true;
        widget.getField('TCDCLIENTE').setValue('S');

        widget.getField('APROVA').checked = true;
        widget.getField('APROVA').setValue('S');
        widget.getField('SOLICITA').checked = true;
        widget.getField('SOLICITA').setValue('S');
    };

    this.setUserPermission = function(dataSource) {
        dataSource = dataSource.map(function(user) {
            user.PERMISSION = "AMBOS";
            if (user.IDAPROVA === 'N') {
                user.PERMISSION = "SOLICITANTE";
            } else if (user.IDSOLICITA === 'N') {
                user.PERMISSION = "APROVADOR";
            }
            return user;
        });
    };
    this.sendInvitation = function() {
        let widget = templateManager.container.getActiveWidget();
        let selectedUsers = widget.getCheckedRows();

        var request = {
            route: 'myo#/crud/qrcode_myorder',
            type: 'FilterData',
            params: [{
                name: 'NRORG',
                operator: '=',
                value: getLocalVar('NRORG')
            }]
        };

        General.callRestEngine(request.route, request.type, request.params).then(function(callback) {

            var DSDNS = callback.dataset.qrcode_myorder[0] ? callback.dataset.qrcode_myorder[0].DSDNS + ';' + getLocalVar('NRORG') : 'nodns';
            let qrCodeContainer = document.createElement('span');
            new QRCode(qrCodeContainer, DSDNS);

            setTimeout(function() {

                General.callRestEngine('myo#/sendInvitationEmailMyOrder', 'Row', {
                    'qrCode': {
                        'data': getLocalVar('qrcode_uri'),
                        'url': DSDNS
                    },
                    'selectedUsers': selectedUsers
                }).then(function(sucess) {

                    ScreenService.showMessage('Convites enviados com sucesso!');

                    widget.dataSource.data.forEach(chr => {
                        chr.__isSelected = false;
                    });
                    widget.reload();
                }, function(error) {
                    ScreenService.showMessage(error.data.error);
                });

            }, 350);
        });
    };

    this.filterBeforeApplyUnidadeCliente = function(args) {
        var filterRow = args.e.preference.FILTER_ROW;
        // filterRow.CDFILIAL = filterRow.NMFILIAL_INSERCAO;
        // filterRow.CDCLIENTE = filterRow.NMRAZSOCCLIE_INSERCAO;
        // delete filterRow.NMFILIAL_INSERCAO;
        // delete filterRow.NMRAZSOCCLIE_INSERCAO;
        // args.e.preference.CUSTOM_FILTER = filterRow;
    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register('usuariosMyOrder_Controller', UsuariosMyOrder_Controller);
});
